unit UltraUtilities;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

{$B-}
{.$DEFINE DEGUG_BKGND_MENU}

interface

uses
 {$IFDEF GX_DEBUG}
  DbugIntf,
  {$ENDIF}
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Themes,
  UxTheme,
  VirtualExplorerTree,
  MPShellUtilities,
  MPCommonObjects,
  VirtualExplorerEasyListview,
  MPDataObject,
  TB2Item,
  TB2ExtItems,
  ShlObj,
  ActiveX,
  CommCtrl,
  SpTBXItem,
  TB2Toolbar,
  MPShellTypes,
  MPCommonUtilities,
  MPThreadManager,
  ActnList,
  TntActnList,
  TntSysUtils,
  TntClasses,
  SpTBXTabs,
  ShellAPI,
  Menus,
  VirtualExplorerEasyListModeView,
  EasyListview,
  Forms,
  VirtualShellNotifier,
  SpTBXSkins,
  UltraUserStrings,
  ATViewer,
  UltraFavTree,
  UnitInPlaceWndInf,
  UltraTaskInf,
  VirtualFileSearch,
  ExtCtrls;

const

  KEY_BACKGND_CONTEXTMENUHANDLERS = 'Directory\Background\shellex\ContextMenuHandlers';
  ULTRA_MENU_MAX = $7FFF;
  ULTRA_MENU_PROPERTIES = ULTRA_MENU_MAX + 1;
  ULTRA_MENU_PASTESHORTCUT = ULTRA_MENU_MAX + 2;
  ULTRA_MENU_PASTE = ULTRA_MENU_MAX + 3;
  ULTRA_MENU_REFRESH = ULTRA_MENU_MAX + 4;
  ULTRA_MENU_ARRANGE = ULTRA_MENU_MAX + 5; 
  ULTRA_MENU_ICONVIEW = ULTRA_MENU_MAX + 6;
  ULTRA_MENU_SMALLICONVIEW = ULTRA_MENU_MAX + 7;
  ULTRA_MENU_LISTVIEW = ULTRA_MENU_MAX + 8;
  ULTRA_MENU_DETAILSVIEW = ULTRA_MENU_MAX + 9;
  ULTRA_MENU_THUMBNAILVIEW = ULTRA_MENU_MAX + 10;
  ULTRA_MENU_TILEVIEW = ULTRA_MENU_MAX + 11;
  ULTRA_MENU_FILMSTRIPVIEW = ULTRA_MENU_MAX + 12;
  ULTRA_MENU_LISTMODE = ULTRA_MENU_MAX + 13;
  ULTRA_MENU_ARRANGE_NAME = ULTRA_MENU_MAX + 14;
  ULTRA_MENU_ARRANGE_SIZE = ULTRA_MENU_MAX + 15;
  ULTRA_MENU_ARRANGE_TYPE = ULTRA_MENU_MAX + 16;
  ULTRA_MENU_ARRANGE_MODIFIED = ULTRA_MENU_MAX + 17;
  ULTRA_MENU_ARRANGE_GROUPS = ULTRA_MENU_MAX + 18;
  ULTRA_MENU_DUALPANE = ULTRA_MENU_MAX + 19;
  ULTRA_MENU_HI = ULTRA_MENU_DUALPANE;

  ID_FILTEROUT_IMAGE = 36;
  ID_FILTERIN_IMAGE = 20;

  TID_FOLDERSIZE = TID_START + 200;

var
  CLSID_NEWMENU: TGUID = '{D969A300-E7FF-11d0-A93B-00A0C90F2719}';

type
  TMenuActionArray = array[1..ULTRA_MENU_HI - ULTRA_MENU_MAX] of TTntAction;
  PPItemIDList = ^PItemIDList;

type
  TUltraColumnModeView = class;

  TGUIDArray = array of TGUID;
  TUltraSpTBXTabItem = class;
  TUltraExplorerEasyListview = class;

  TUltraFileSizeCompare = (
    fscAtLeast,      // size is greater then X
    fcsNoMore        // size is less then X
  );

  TUltraSearchAttribMask = (
    samFileMask,      // Use FileNameMask
    samContentMask,   // Use FileContentMask
    samModifyMask,    // Use ModifyDateMin, ModifyDateMax
    samSizeMask       // Use Size, SizeCompare
  );
  TUltraSearchAttribMasks = set of TUltraSearchAttribMask;

  TUltraDateSearchType = (
    dstModified,
    dstCreated,
    dstAccessed
  );

  PUltraSearchAttribs = ^TUltraSearchAttribs;
  TUltraSearchAttribs = record
    Masks: TUltraSearchAttribMasks;
    FileNameMask,
    FileContentMask: WideString;
    SearchFolders: TTntStringList;
    DateMin,
    DateMax: TFileTime;
    DateSearch: TUltraDateSearchType;
    Size: Int64;
    SizeCompare: TUltraFileSizeCompare;
    RecurseSubFolder: Boolean;
    CaseSensitive: Boolean;
    AdvOptions: TVirtualSearchAttribs;
  end;

  TUltraFilterDir = (
    ufdFilterIn,     // The filter includes the file type of regular expression match
    ufdFilterOut     // The filter excludes the file type of regular expression match
  );

  TUltraFilterKind = (
    ufkUserDefined, // user definded filter.  Found in the second group in the LV
    ufkFolders      // UE defined Folders filter
  );

  TViewerOpen = procedure(Viewer: WideString) of object;
  TViewerOpenFile = function: WideString of object;
  TCmdLineSetPath = procedure(NS: TNamespace) of object;

  // ***************************************************************************
  // ***************************************************************************

  TUltraItemState = (
    uisThreadLoading,
    uisThreadLoaded
  );
  TUltraItemStates = set of TUltraItemState;

  // ***************************************************************************
  //  Thread request to extract folder size
  // ***************************************************************************
  TUltraFolderSizeThreadRequest = class(TPIDLThreadRequest)
  private
    FFolderSize: Int64;
  public
    function HandleRequest: Boolean; override;
    property FolderSize: Int64 read FFolderSize write FFolderSize;
  end;

  TUltraExplorerItem = class(TExplorerItem)
  private
    FFolderSize: Int64;
    FUltraStates: TUltraItemStates;
  public
    constructor Create(ACollection: TEasyCollection); override;
    property FolderSize: Int64 read FFolderSize write FFolderSize;
    property UltraStates: TUltraItemStates read FUltraStates write FUltraStates;
  end;

  // ***************************************************************************
  // Derivation of the ListModeEasyListview (Columns Mode) that help maintain various
  // calls to it without caring if it is in the Column Mode or if it is in the
  // normal Explorer Listview modes.
  // This is just a single column Listview if in Columns Mode that is used in the
  // TUltraListModeView class below.
  // Also is aware of the Viewer.
  // ***************************************************************************
  TUltraExplorerEasyListview = class(TColumnModeEasyListview, IOLEWindow, IShellBrowser, IUltraMenuCallback)
  private
    FCmdLineCurerntPath: PPItemIDList;
    FCmdLineSetPath: TCmdLineSetPath;
    FDraggingFromBackground: Boolean;
    FEnableGesture: Boolean;
    FFolderSizeMax: Int64;
    FLockChangeLink: Boolean;
    FOpenViewer: TViewerOpen;
    FChangeNotifierEnabled: boolean;
    FViewerOpenFile: TViewerOpenFile;
    function GetColumnModeView: TUltraColumnModeView;
    function GetFileObjects: TFileObjects;
    function GetShowInactiveEx: Boolean;
    procedure SetColumnModeView(const Value: TUltraColumnModeView);
    procedure SetEnableGesture(const Value: Boolean);
    procedure SetFileObjects(const Value: TFileObjects);
    procedure SetFolderSizeMax(const Value: Int64);
    procedure SetChangeNotifierEnabled(const Value: boolean);
    procedure SetShowInactiveEx(const Value: Boolean);
  protected
    // IOLEWindow
    function GetWindow(out wnd: HWnd): HResult; stdcall;
    function ContextSensitiveHelp(fEnterMode: BOOL): HResult; stdcall;

    // IShellBrowser
    function InsertMenusSB(hMenuShared: HMENU; out MenuWidths: TOleMenuGroupWidths): HResult; stdcall;
    function SetMenuSB(hMenuShared: HMENU; hOleMenuReserved: HOLEMENU; hwndActiveObject: HWND): HResult; stdcall;
    function RemoveMenusSB(hMenuShared: HMENU): HResult; stdcall;
    function SetStatusTextSB(StatusText: POleStr): HResult; stdcall;
    function EnableModelessSB(Enable: BOOL): HResult; stdcall;
    function TranslateAcceleratorSB(Msg: PMsg; ID: Word): HResult; stdcall;
    function BrowseObject(pidl: PItemIDList; flags: UINT): HResult; stdcall;
    function GetViewStateStream(Mode: DWORD; out Stream: IStream): HResult; stdcall;
    function GetControlWindow(ID: UINT; out Wnd: HWND): HResult; stdcall;
    function SendControlMsg(ID, Msg: UINT; wParm: WPARAM; lParm: LPARAM; var Rslt: LResult): HResult; stdcall;
    function QueryActiveShellView(var ShellView: IShellView): HResult; stdcall;
    function OnViewWindowActive(var ShellView: IShellView): HResult; stdcall;
    function SetToolbarItems(TBButton: PTBButton; nButtons, uFlags: UINT): HResult; stdcall;

    //IUltraCallback
    procedure BrowseToByCallback(PIDL: PItemIDList; NewTab: Boolean);

    function DoItemCompare(Column: TEasyColumn; Group: TEasyGroup; Item1: TEasyItem; Item2: TEasyItem): Integer; override;
    procedure CheckFocus; override;
    procedure CMRecreateWnd(var Msg: TMessage); message CM_RECREATEWND;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DoEnumFinished; override;
    procedure DoExplorerItemClass(var ItemClass: TExplorerItemClass); override;
    procedure DoItemEditBegin(Item: TEasyItem; var Column: Integer; var Allow: Boolean); override;
    procedure DoItemGetCaption(Item: TEasyItem; Column: Integer; var ACaption: WideString); override;
    procedure DoItemSelectionsChanged; override;
    procedure DoItemSetCaption(Item: TEasyItem; Column: Integer; const Caption: WideString); override;
    procedure DoItemStructureChange; override;
    procedure DoRootChange; override;
    procedure DoRootChanging(const NewRoot: TRootFolder; Namespace: TNamespace; var Allow: Boolean); override;
    procedure DoThreadCallback(var Msg: TWMThreadRequest); override;
    procedure HandleMouseUp(Button: TCommonMouseButton; Msg: TWMMouse); override;
    procedure WMDestroy(var Msg: TMessage); message WM_DESTROY;
    procedure WMGetIShellBrowser(var Msg: TMessage); message CWM_GETISHELLBROWSER;
    procedure WMKillFocus(var Msg: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMMouseActivate(var Msg: TWMMouseActivate); message WM_MOUSEACTIVATE;
    procedure WMNCCalcSize(var Msg: TWMNCCalcSize); message WM_NCCALCSIZE;
    property DraggingFromBackground: Boolean read FDraggingFromBackground write FDraggingFromBackground;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddCustomItemEx(Group: TEasyGroup; NS: TNamespace): TExplorerItem;
    function FindItemByPathEx(Path: WideString): TExplorerItem;
    function FirstSelection: TExplorerItem;
    function ItemCountEx: Integer;
    function NextSelection(Item: TExplorerItem): TExplorerItem;
    function SelectedToNamespaceArrayEx: TNamespaceArray;
    function SelectionCount: Integer;
    function ValidRootNamespaceEx: Boolean;
    procedure BrowseToPrevLevelEx;
    procedure ChangeLinkChanging(Server: TObject; NewPIDL: PItemIDList); override;
    procedure ChangeLinkDispatch; override;
    procedure CutToClipboard; override;
    procedure EndUpdate(Invalidate: Boolean = True); override;
    procedure FolderSizeClear;
    procedure FolderSizeLoadThread(SelectedOnly, Force: Boolean);
    procedure RebuildEx;
    procedure RefreshEx;
    procedure SelectedFilesDelete; override;
    procedure SelectionClearAll;
    procedure SelectionInvert;
    procedure SelectionSelectAll;
    procedure SetFocusEx;
    property CmdLineCurrentPath: PPItemIDList read FCmdLineCurerntPath write FCmdLineCurerntPath;
    property CmdLineSetPath: TCmdLineSetPath read FCmdLineSetPath write FCmdLineSetPath;
    property ColumnModeView: TUltraColumnModeView read GetColumnModeView write SetColumnModeView;
    property EnableGesture: Boolean read FEnableGesture write SetEnableGesture;
    property FolderSizeMax: Int64 read FFolderSizeMax write SetFolderSizeMax;
    property LockChangeLink: Boolean read FLockChangeLink write FLockChangeLink;
    property OpenViewer: TViewerOpen read FOpenViewer write FOpenViewer;
    property ChangeNotifierEnabled: boolean read FChangeNotifierEnabled write SetChangeNotifierEnabled;
    property ShowInactiveEx: Boolean read GetShowInactiveEx write SetShowInactiveEx;
    property ViewerOpenFile: TViewerOpenFile read FViewerOpenFile write FViewerOpenFile;
  published
    property FileObjects: TFileObjects read GetFileObjects write SetFileObjects;
  end;

  // ***************************************************************************
  // Derivation of the TVirtualListModeView that sets the object as Transient so it will
  // not be streamed into the DFM file.
  // ***************************************************************************
  TUltraColumnModeView = class(TVirtualColumnModeView)
  private
    FEnableGesture: Boolean;
    FChangeNotifierEnabled: Boolean;
    procedure SetEnableGesture(const Value: Boolean);
    procedure SetChangeNotifierEnabled(const Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property EnableGesture: Boolean read FEnableGesture write SetEnableGesture;
    property ChangeNotifierEnabled: Boolean read FChangeNotifierEnabled write SetChangeNotifierEnabled;
  end;

  // ***************************************************************************
  // ***************************************************************************
  TUltraInPlacePanel = class(TPanel)
  private
    FInPlaceWindow: IUltraInPlaceWindow;
    procedure SetInPlaceWindow(const Value: IUltraInPlaceWindow);
  protected
    procedure WMEraseBkgnd(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property InPlaceWindow: IUltraInPlaceWindow read FInPlaceWindow write SetInPlaceWindow;
  end;

  // ***************************************************************************
  // TabItem derivations that contains a PIDL to associate with it
  // ***************************************************************************
  TUltraSpTBXTabItem = class(TSpTBXTabItem)
  private
    FHistory: TMemoryStream;               // A memory image for the History Stack that is associated with the Tab, valid only after the tab is switched away from
    FHistoryIndex: Integer;                // The Current Index for the History stack, valid only after the tab is switched away from
    FInPlaceTab: Boolean;                  // Set True if the Tab is a container for an InPlaceWindow
    FInplaceWindow: IUltraInPlaceWindow;   // Interface to the instance of the InPlaceWindow that is shown in the Tab
    FMRU: TMemoryStream;                   // A memory image for the Most Recently Used Stack that is associated with the Tab, valid only after the tab is switched away from
    FPIDL: PItemIDList;                    // The Folder that this tab is a view of
    FQuickFilterContents: WideString;
    FSelected: TMemoryStream;              // A memory image of the items that are selected, valid only after the tab is switched away from
    procedure SetInPlaceWindow(const Value: IUltraInPlaceWindow);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property History: TMemoryStream read FHistory write FHistory;
    property HistoryIndex: Integer read FHistoryIndex write FHistoryIndex;
    property InPlaceTab: Boolean read FInPlaceTab;
    property InPlaceWindow: IUltraInPlaceWindow read FInplaceWindow write SetInPlaceWindow;
    property MRU: TMemoryStream read FMRU write FMRU;
    property PIDL: PItemIDList read FPIDL write FPIDL;
    property QuickFilterContents: WideString read FQuickFilterContents write FQuickFilterContents;
    property Selected: TMemoryStream read FSelected write FSelected;
  end;

  // ***************************************************************************
  // TUltraContextMenuMap -
  //   Maintains a mapping between which IContextMenu2 interface and menu items are
  // related.  This is necessary to dispatch the custom draw menu messages to the
  // correct interface
  // ***************************************************************************
  PUltraContextMenuMap = ^TUltraContextMenuMap;
  TUltraContextMenuMap = record
    Menu: IContextMenu;     // The ContextMenu interfaces accociated with the
    ControlID: Integer;
    SubMenu: HMenu;
    Offset: Integer;        // The first menu item ID used to generate the IContextMenu
  end;

  // ***************************************************************************
  // TUltraViewBkGndContextMenu -
  //  Handles the mapping between IContextMenu2 interfaces and context menu items
  // for custom drawn shell menu handlers
  // ***************************************************************************
  TUltraViewBkGndContextMenu = class
  private
    FActions: TMenuActionArray;
    FExcludeList: TGUIDArray;
    FItemList: TList;
    FMenuIDStore: TList;
    function GetActions(Index: Integer): TTntAction;
    function GetItems(Index: Integer): PUltraContextMenuMap;
    procedure SetActions(Index: Integer; Value: TTntAction);
    procedure SetItems(Index: Integer; Value: PUltraContextMenuMap);
  protected
    function CreateArrangeSubMenu: HMenu;
    function CreateSubMenuView: HMenu;
    function ItemInMenuStore(ItemID: Cardinal): Boolean;
    procedure CompareMenuStore(Menu: HMenu; ContextMenu: IContextMenu; Offset: Integer);
    procedure FillSeparatorList(Menu: HMenu; Separators: TList);
    procedure InjectStandardItems(Menu: HMenu; InjectPropSeparator: Boolean);
    procedure InsertSeparator(Menu: HMenu; InsertIndex: Integer);
    function LoadMenuItemA(Caption: String; Enabled, Checked: Boolean; SubMenu: HMenu; ItemID: Cardinal): TMenuItemInfoA;
    function LoadMenuItemW(Caption: WideString; Enabled, Checked: Boolean; SubMenu: HMenu; ItemID: Cardinal): TMenuItemInfoW;
    procedure RemoveExcludedGUIDs(var GUIDs: TGUIDArray);
    procedure StoreMenuIDs(Menu: HMenu);
    procedure ValidateMenuSeparators(Menu: HMenu);
    property ItemList: TList read FItemList write FItemList;
    property MenuIDStore: TList read FMenuIDStore write FMenuIDStore;
  public
    constructor Create;
    destructor Destroy; override;
    function FindMenuByControl(ControlID: Integer): IContextMenu; overload;
    function FindMenuBySubMenu(SubMenu: HMenu): IContextMenu;
    function FindMenuMapByID(ItemID: Integer): PUltraContextMenuMap;
    function FindMenuMapBySubMenu(SubMenu: HMenu): PUltraContextMenuMap;
    procedure Add(const Menu: IContextMenu; ItemID: Cardinal; SubMenu: HMenu; Offset: Integer);
    procedure Clear;
    procedure DispatchToMenu(var Msg: TMessage);
    procedure LoadFromStream(S: TStream);
    procedure SaveToStream(S: TStream);
    procedure Show(Browser: IShellBrowser; ScreenPt: TPoint; Window: HWnd; NS: TNamespace; var IsNewMenuCommand: Boolean);
    property Actions[Index: Integer]: TTntAction read GetActions write SetActions;
    property ExcludeList: TGUIDArray read FExcludeList write FExcludeList;
    property Items[Index: Integer]: PUltraContextMenuMap read GetItems write SetItems;
  published
  end;

  // ***************************************************************************
  // Data structure for the Favorites VirtualTreeview objects
  // ***************************************************************************
  PUltraFavoritesRec = ^TUltraFavoritesRec;
  TUltraFavoritesRec = record
    NS: TNamespace;
    Caption: WideString;
    ImageIndex,
    ImageIndexOpen: Integer
  end;

  // ***************************************************************************
  // Class that takes an array of TNamespaces and sorts them into groups that
  // have the same parent PIDL. Each group is a TList of the passed namespaces
  // that are references to the originals, the class does not own the objects that
  // are contained in the Lists but it does own the Lists.
  // ***************************************************************************
  TNamespaceGrouping = class
  private
    FGroups: TList;
    function GetCount: Integer;
    function GetGroups(Index: Integer): TVirtualNameSpaceList;
    procedure SetGroups(Index: Integer; Value: TVirtualNameSpaceList);
  public
    constructor Create;
    destructor Destroy; override;
    function AddGroup: TVirtualNamespaceList;
    function GroupToArray(Group: TVirtualNamespaceList): TNamespaceArray;
    procedure Clear;
    procedure SortIntoGroup(NS: TNamespace);
    property Count: Integer read GetCount;
    property Groups[Index: Integer]: TVirtualNameSpaceList read GetGroups write SetGroups; default;
  end;

  // ***************************************************************************
  // EasyItem derivation that adds information about each filter object
  // ***************************************************************************
  TUltraFilterItem = class(TEasyItemStored)
  private
    FFilterKind: TUltraFilterKind;
    function GetFilterEnabled: Boolean;
    function GetMask: WideString;
    procedure SetFilterEnabled(const Value: Boolean);
  public
    constructor Create(ACollection: TEasyCollection); override;
    procedure LoadFromStream(S: TStream; var Version: Integer); override;
    procedure SaveToStream(S: TStream; Version: Integer); override;
    property FilterEnabled: Boolean read GetFilterEnabled write SetFilterEnabled;
    property FilterKind: TUltraFilterKind read FFilterKind write FFilterKind;
    property Mask: WideString read GetMask;
  end;


  // ***************************************************************
  // ***************************************************************
  TUltraViewReportItem = class(TEasyViewReportItem)
  public
    procedure PaintBefore(Item: TEasyItem; Column: TEasyColumn; const Caption: WideString; ACanvas: TCanvas; RectArray: TEasyRectArrayObject; var Handled: Boolean); override;
  end;

  // ***************************************************************
  // ***************************************************************
  TUltraFileSearch = class(TVirtualFileSearch)
  private
    FSearchData: PUltraSearchAttribs;
  public
    destructor Destroy; override;
    function Run: Boolean; override;
    procedure Stop; override;
    property SearchData: PUltraSearchAttribs read FSearchData write FSearchData;
  end;

function DataObjectSupportsShell(const DataObj: IDataObject): Boolean;
procedure EnableToolbarItems(Toolbar: TTBToolbar; Enable: Boolean);
procedure FindBkGndContextMenus(var GUIDs: TGUIDArray);
function UltraSelectDirectory(Wnd: HWnd; InitialDir: PItemIDList; var OutPath: PItemIDList): Boolean;
procedure SetTBItemCaption(Item: TTBCustomItem; Caption: WideString);
procedure SetTBItemHint(Item: TTBCustomItem; Hint: WideString);

implementation

uses
  TypInfo,
  Registry,
  UltraOptions;

type
  TCustomEasyListviewHack = class(TCustomEasyListview);

var
  InitialDirPIDL: PItemIDList; // for use with SHBrowseForFolder;

procedure SetTBItemHint(Item: TTBCustomItem; Hint: WideString);
// Set the unicode caption to the Item if it has a valid
// WideString Caption property.
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(Item, 'Hint', [tkWString]);
  if PropInfo = nil then
    Item.Caption := Hint
  else
    SetWideStrProp(Item, PropInfo, Hint);
end;

procedure SetTBItemCaption(Item: TTBCustomItem; Caption: WideString);
// Set the unicode caption to the Item if it has a valid
// WideString Caption property.
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(Item, 'Caption', [tkWString]);
  if PropInfo = nil then
    Item.Caption := Caption
  else
    SetWideStrProp(Item, PropInfo, Caption);
end;

procedure FindBkGndContextMenus(var GUIDs: TGUIDArray);
var
  Reg: TRegistry;
  Vals: TStringList;
  i: Integer;
  s: string;
  GUID: TGUID;
begin
  Reg := TRegistry.Create;
  Vals := TStringList.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey(KEY_BACKGND_CONTEXTMENUHANDLERS, False) then
    begin
      try
        Reg.GetKeyNames(Vals);
      finally
        Reg.CloseKey;
      end;
      SetLength(GUIDs, Vals.Count);
      for i := 0 to Vals.Count - 1 do
      begin
        if Reg.OpenKey(KEY_BACKGND_CONTEXTMENUHANDLERS + '\' + Vals[i], False) then
        begin
          try
            s := Reg.ReadString('');
          finally
            Reg.CloseKey;
          end;
          s := Trim(s);
          if Succeeded(CLSIDFromString(PWideChar(WideString(s)), GUID)) then
            GUIDs[i] := GUID
          else
           GUIDs[i] := GUID_NULL
        end
      end
    end
  finally
    Vals.Free;
    Reg.Free;
  end
end;

function BrowseCallbackProc(hwnd: HWND; uMsg: UINT; lParam: LPARAM; lpData: LPARAM): Integer; stdcall;
begin
  Result := 0;
  case uMsg of
    BFFM_INITIALIZED:
      begin
        if IsUnicode then
          Result := SendMessageW(hwnd, BFFM_SETSELECTIONW, 0, LongInt( InitialDirPIDL))
        else
          Result := SendMessageA(hwnd, BFFM_SETSELECTIONA, 0, LongInt( InitialDirPIDL))
      end;
  end
end;

function UltraSelectDirectory(Wnd: HWnd; InitialDir: PItemIDList; var OutPath: PItemIDList): Boolean;
var
  BrowseInfoA: TBrowseInfoA;
  BrowseInfoW: TBrowseInfoW;
  WindowList: Pointer;
  OldErrorMode: Cardinal;
  BufferW: PWideChar;
  BufferA: PChar;
begin
  OutPath := nil;
  if IsUnicode then
  begin
    FillChar(BrowseInfoW, SizeOf(BrowseInfoW), #0);
    BufferW := PIDLMgr.Malloc.Alloc(MAX_PATH*2);
    BrowseInfoW.hwndOwner := Wnd;
    BrowseInfoW.lpszTitle := 'Select a Folder for the New Tab';
    BrowseInfoW.pszDisplayName := BufferW;
    BrowseInfoW.pidlRoot := PIDLMgr.CopyPIDL(DesktopFolder.AbsolutePIDL);
    BrowseInfoW.ulFlags := BIF_NEWDIALOGSTYLE or BIF_RETURNONLYFSDIRS;
    BrowseInfoW.lpfn := BrowseCallbackProc;
    InitialDirPIDL := PIDLMgr.CopyPIDL(InitialDir);
    try
      WindowList := DisableTaskWindows(0);
      OldErrorMode := SetErrorMode(SEM_FAILCRITICALERRORS);
      try
        OutPath := ShBrowseForFolderW(BrowseInfoW);
        Result := Assigned(OutPath)
      finally
        SetErrorMode(OldErrorMode);
        EnableTaskWindows(WindowList);
      end;
    finally
      PIDLMgr.FreeOLEStr(BufferW);
      PIDLMgr.FreeAndNilPIDL(InitialDirPIDL);
      PIDLMgr.FreePIDL(BrowseInfoW.pidlRoot);
    end
  end else
  begin
    FillChar(BrowseInfoA, SizeOf(BrowseInfoA), #0);
    BufferA := PIDLMgr.Malloc.Alloc(MAX_PATH);
    BrowseInfoA.hwndOwner := Wnd;
    BrowseInfoA.lpszTitle := 'Select a Folder for the New Tab';
    BrowseInfoA.pszDisplayName := BufferA;
    BrowseInfoA.pidlRoot := PIDLMgr.CopyPIDL(DesktopFolder.AbsolutePIDL);
    BrowseInfoA.ulFlags := BIF_NEWDIALOGSTYLE or BIF_RETURNONLYFSDIRS;
    BrowseInfoA.lpfn := BrowseCallbackProc;
    InitialDirPIDL := PIDLMgr.CopyPIDL(InitialDir);
    try
      WindowList := DisableTaskWindows(0);
      OldErrorMode := SetErrorMode(SEM_FAILCRITICALERRORS);
      try
        OutPath := ShBrowseForFolderA(BrowseInfoA);
        Result := Assigned(OutPath)
      finally
        SetErrorMode(OldErrorMode);
        EnableTaskWindows(WindowList);
      end;
    finally
      PIDLMgr.Malloc.Free(BufferA);
      PIDLMgr.FreeAndNilPIDL(InitialDirPIDL);
      PIDLMgr.FreePIDL(BrowseInfoA.pidlRoot);
    end
  end
end;

function DataObjectSupportsShell(const DataObj: IDataObject): Boolean;
begin
  Result := False;
  if Assigned(DataObj) then
  begin
    Result := Succeeded(DataObj.QueryGetData(HDropFormat)) or
      Succeeded(DataObj.QueryGetData(ShellIDListFormat))
  end
end;

procedure EnableToolbarItems(Toolbar: TTBToolbar; Enable: Boolean);

    procedure RunItems(Item: TTBCustomItem; Enable: Boolean);
    var
      i: Integer;
    begin
      Item.Enabled := Enable;
      for i := 0 to Item.Count - 1 do
        RunItems(Item[i], Enable)
    end;

begin
  RunItems(Toolbar.Items, Enable)
end;

{ TUltraViewBkGndContextMenu }
constructor TUltraViewBkGndContextMenu.Create;
begin
  inherited Create;
  ItemList := TList.Create;
  MenuIDStore := TList.Create;
end;

destructor TUltraViewBkGndContextMenu.Destroy;
begin
  Clear;
  ItemList.Free;
  MenuIDStore.Free;
  inherited Destroy;
end;

function TUltraViewBkGndContextMenu.CreateArrangeSubMenu: HMenu;
var
  Action: TTntAction;
begin
  Result := CreatePopupMenu;
  if IsUnicode then
  begin
    Action := Actions[ULTRA_MENU_ARRANGE_GROUPS - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_GROUPS));
    InsertMenuItemW(Result, 0, True, LoadMenuItemW('-', True, False, 0, $FFFF));
    Action := Actions[ULTRA_MENU_ARRANGE_MODIFIED - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_MODIFIED));
    Action := Actions[ULTRA_MENU_ARRANGE_TYPE - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_TYPE));
    Action := Actions[ULTRA_MENU_ARRANGE_SIZE - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_SIZE));
    Action := Actions[ULTRA_MENU_ARRANGE_NAME - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_NAME));
  end else
  begin
    Action := Actions[ULTRA_MENU_ARRANGE_GROUPS - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_GROUPS));
    InsertMenuItemA(Result, 0, True, LoadMenuItemA('-', True, False, 0, $FFFF));
    Action := Actions[ULTRA_MENU_ARRANGE_MODIFIED - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_MODIFIED));
    Action := Actions[ULTRA_MENU_ARRANGE_TYPE - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_TYPE));
    Action := Actions[ULTRA_MENU_ARRANGE_SIZE - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_SIZE));
    Action := Actions[ULTRA_MENU_ARRANGE_NAME - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ARRANGE_NAME));
  end 
end;

function TUltraViewBkGndContextMenu.CreateSubMenuView: HMenu;
var
  Action: TTntAction;
begin
  Result := CreatePopupMenu;
  if IsUnicode then
  begin
    Action := Actions[ULTRA_MENU_DUALPANE - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_DUALPANE));
    InsertMenuItemW(Result, 0, True, LoadMenuItemW('-', Action.Enabled, False, 0, $FFFF));
    Action := Actions[ULTRA_MENU_LISTMODE - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_LISTMODE));
    Action := Actions[ULTRA_MENU_THUMBNAILVIEW - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_THUMBNAILVIEW));
    Action := Actions[ULTRA_MENU_TILEVIEW - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_TILEVIEW));
    Action := Actions[ULTRA_MENU_DETAILSVIEW - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_DETAILSVIEW));
    Action := Actions[ULTRA_MENU_LISTVIEW - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_LISTVIEW));
    Action := Actions[ULTRA_MENU_SMALLICONVIEW - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_SMALLICONVIEW));
    Action := Actions[ULTRA_MENU_ICONVIEW - ULTRA_MENU_MAX];
    InsertMenuItemW(Result, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ICONVIEW));
  end else
  begin
    Action := Actions[ULTRA_MENU_DUALPANE - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_DUALPANE));
    InsertMenuItemA(Result, 0, True, LoadMenuItemA('-', Action.Enabled, False, 0, $FFFF));
    Action := Actions[ULTRA_MENU_LISTMODE - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_LISTMODE));
    Action := Actions[ULTRA_MENU_THUMBNAILVIEW - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_THUMBNAILVIEW));
    Action := Actions[ULTRA_MENU_TILEVIEW - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_TILEVIEW));
    Action := Actions[ULTRA_MENU_DETAILSVIEW - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_DETAILSVIEW));
    Action := Actions[ULTRA_MENU_LISTVIEW - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_LISTVIEW));
    Action := Actions[ULTRA_MENU_SMALLICONVIEW - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_SMALLICONVIEW));
    Action := Actions[ULTRA_MENU_ICONVIEW - ULTRA_MENU_MAX];
    InsertMenuItemA(Result, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_ICONVIEW));
  end
end;

function TUltraViewBkGndContextMenu.FindMenuByControl(ControlID: Integer): IContextMenu;
var
  i: Integer;
begin
  Result := nil;
  i := 0;
  while (i < ItemList.Count) and not Assigned(Result) do
  begin
    if Items[i]^.ControlID = ControlID then
      Result := Items[i]^.Menu;
    Inc(i)
  end
end;

function TUltraViewBkGndContextMenu.FindMenuBySubMenu(SubMenu: HMenu): IContextMenu;
var
  i: Integer;
begin
  Result := nil;
  i := 0;
  while (i < ItemList.Count) and not Assigned(Result) do
  begin
    if Items[i]^.SubMenu = SubMenu then
      Result := Items[i]^.Menu;
    Inc(i)
  end
end;

function TUltraViewBkGndContextMenu.FindMenuMapByID(ItemID: Integer): PUltraContextMenuMap;
var
  i: Integer;
begin
  Result := nil;
  i := 0;
  while (i < ItemList.Count) and not Assigned(Result) do
  begin
    if Items[i]^.ControlID = ItemID then
      Result := Items[i];
    Inc(i)
  end
end;

function TUltraViewBkGndContextMenu.GetActions(Index: Integer): TTntAction;
begin
  Result := FActions[Index]
end;

function TUltraViewBkGndContextMenu.GetItems(Index: Integer): PUltraContextMenuMap;
begin
  Result := PUltraContextMenuMap( ItemList[Index])
end;

function TUltraViewBkGndContextMenu.ItemInMenuStore(ItemID: Cardinal): Boolean;
var
  i: Integer;
begin
  Result := False;
  i := 0;
  while not Result and (i < MenuIDStore.Count) do
  begin
    Result := MenuIDStore[i] = Pointer(ItemID);
    Inc(i)
  end
end;

procedure TUltraViewBkGndContextMenu.Add(const Menu: IContextMenu; ItemID: Cardinal; SubMenu: HMenu; Offset: Integer);
var
  Item: PUltraContextMenuMap;
begin
  New(Item);
  Item^.ControlID := ItemID;
  Item^.Menu := Menu;
  Item^.SubMenu := SubMenu;
  Item^.Offset := Offset;
  ItemList.Add(Item)
end;

procedure TUltraViewBkGndContextMenu.Clear;
var
  i: Integer;
begin
  try
    for i := ItemList.Count - 1 downto 0  do
    begin
      PUltraContextMenuMap( ItemList[i]).Menu := nil; // Release the interface
      Dispose( PUltraContextMenuMap(ItemList[i]));
    end
  finally
    ItemList.Clear
  end
end;

procedure TUltraViewBkGndContextMenu.CompareMenuStore(Menu: HMenu; ContextMenu: IContextMenu; Offset: Integer);
var
  i: Integer;
  MenuInfoA: TMenuItemInfoA;
  MenuInfoW: TMenuItemInfoW;
begin
  for i := 0 to GetMenuItemCount(Menu) - 1 do
  begin
    if IsUnicode then
    begin
      FillChar(MenuInfoW, SizeOf(MenuInfoW), #0);
      MenuInfoW.cbSize := SizeOf(MenuInfoW);
      MenuInfoW.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU	or MIIM_STATE	or MIIM_CHECKMARKS or MIIM_DATA;
      if GetMenuItemInfoW(Menu, i, True, MenuInfoW) then
      begin
        if not ItemInMenuStore(MenuInfoW.wID) then
          Add(ContextMenu, MenuInfoW.wID, MenuInfoW.hSubMenu, Offset);
      end
    end else
    begin
      FillChar(MenuInfoA, SizeOf(MenuInfoA), #0);
      MenuInfoA.cbSize := SizeOf(MenuInfoA);
      MenuInfoA.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU	or MIIM_STATE	or MIIM_CHECKMARKS or MIIM_DATA;
      if GetMenuItemInfoA(Menu, i, True, MenuInfoA) then
      begin
        if not ItemInMenuStore(MenuInfoA.wID) then
          Add(ContextMenu, MenuInfoA.wID, MenuInfoA.hSubMenu, Offset);
      end
    end
  end
end;

procedure TUltraViewBkGndContextMenu.DispatchToMenu(var Msg: TMessage);
var
  ContextMenu: IContextMenu;
  Menu2: IContextMenu2;
  OldMenuCount, NewMenuCount: Integer;
  hSubMenu: HMenu;
  Map: PUltraContextMenuMap;
begin
  // The only time the ItemList has items is when the context menu is show so
  // we can be sure that when it is <> 0 we are needing to dispatch them to the
  // IContextMenu2 instances.
  if ItemList.Count > 0 then
  begin
    case Msg.Msg of
      WM_DRAWITEM:
        begin
          // First make sure it is a menu message
          if TWMDrawItem(Msg).Ctl = 0 then
          begin
            ContextMenu := FindMenuByControl(TWMDrawItem(Msg).DrawItemStruct.itemID);
            if Assigned(ContextMenu) then
            // There should be no excuse for this query to fail.  If we are in this
            // message then it must be owner draw and support IContextMenu2
              if ContextMenu.QueryInterface(IContextMenu2, Menu2) = S_OK then
                Menu2.HandleMenuMsg(Msg.Msg, Msg.WParam, Msg.LParam);
          end
        end;
      WM_INITMENUPOPUP:
        begin
          hSubMenu := TWMInitMenuPopup(Msg).MenuPopup;
          OldMenuCount := GetMenuItemCount(hSubMenu);
          Map := FindMenuMapBySubMenu(hSubMenu);
          StoreMenuIDs(hSubMenu);
          if Assigned(Map) then
          begin
            ContextMenu := Map.Menu;
            // There should be no excuse for this query to fail.  If we are in this
            // message then it must be owner draw and support IContextMenu2
            if ContextMenu.QueryInterface(IContextMenu2, Menu2) = S_OK then
              Menu2.HandleMenuMsg(Msg.Msg, Msg.WParam, Msg.LParam);
            NewMenuCount := GetMenuItemCount(hSubMenu);
            // The handler added new menu items to the list
            if NewMenuCount > OldMenuCount then
              CompareMenuStore(hSubMenu, ContextMenu, Map.Offset)
          end;
        end;
      WM_MEASUREITEM:
        begin
          ContextMenu := FindMenuByControl(TWMMeasureItem(Msg).MeasureItemStruct.itemID);
            if Assigned(ContextMenu) then
              // There should be no excuse for this query to fail.  If we are in this
              // message then it must be owner draw and support IContextMenu2
              if ContextMenu.QueryInterface(IContextMenu2, Menu2) = S_OK then
                Menu2.HandleMenuMsg(Msg.Msg, Msg.WParam, Msg.LParam);
        end;
    end
  end
end;

procedure TUltraViewBkGndContextMenu.FillSeparatorList(Menu: HMenu; Separators: TList);
var
  i: Integer;
  MenuInfoA: TMenuItemInfoA;
  MenuInfoW: TMenuItemInfoW;
begin
  Separators.Clear;
  for i := 0 to GetMenuItemCount(Menu) - 1 do
  begin
    if IsUnicode then
    begin
      FillChar(MenuInfoW, SizeOf(MenuInfoW), #0);
      MenuInfoW.cbSize := SizeOf(MenuInfoW);
      MenuInfoW.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU or MIIM_STATE or MIIM_CHECKMARKS or MIIM_DATA;
      if GetMenuItemInfoW(Menu, i, True, MenuInfoW) then
      begin
        if (MenuInfoW.fType and MFT_SEPARATOR = MFT_SEPARATOR) and (MenuInfoW.hSubMenu = 0) then
          Separators.Add(Pointer(i))
        else
          Separators.Add(Pointer(-1))
      end
    end
    else
    begin
      FillChar(MenuInfoA, SizeOf(MenuInfoA), #0);
      MenuInfoA.cbSize := SizeOf(MenuInfoA);
      MenuInfoA.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU or MIIM_STATE or MIIM_CHECKMARKS or MIIM_DATA;
      if GetMenuItemInfoA(Menu, i, True, MenuInfoA) and (MenuInfoA.hSubMenu = 0) then
      begin
        if MenuInfoA.fType and MFT_SEPARATOR = MFT_SEPARATOR then
          Separators.Add(Pointer(i))
        else
          Separators.Add(Pointer(-1))
      end
    end
  end;
end;

function TUltraViewBkGndContextMenu.FindMenuMapBySubMenu(SubMenu: HMenu): PUltraContextMenuMap;
var
  i: Integer;
begin
  Result := nil;
  i := 0;
  while (i < ItemList.Count) and not Assigned(Result) do
  begin
    if Items[i]^.SubMenu = SubMenu then
      Result := Items[i];
    Inc(i)
  end
end;

procedure TUltraViewBkGndContextMenu.InjectStandardItems(Menu: HMenu; InjectPropSeparator: Boolean);
var
  Action: TTntAction;
begin
  if IsUnicode then
  begin
    Action := Actions[ULTRA_MENU_PROPERTIES - ULTRA_MENU_MAX];
    InsertMenuItemW(Menu, GetMenuItemCount(Menu), True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_PROPERTIES));
    if InjectPropSeparator then
      InsertSeparator(Menu, 0);
    Action := Actions[ULTRA_MENU_PASTESHORTCUT - ULTRA_MENU_MAX];
    InsertMenuItemW(Menu, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_PASTESHORTCUT));
    Action := Actions[ULTRA_MENU_PASTE - ULTRA_MENU_MAX];
    InsertMenuItemW(Menu, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_PASTE));
    InsertSeparator(Menu, 0);
    Action := Actions[ULTRA_MENU_REFRESH - ULTRA_MENU_MAX];
    InsertMenuItemW(Menu, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_REFRESH));
    Action := Actions[ULTRA_MENU_ARRANGE - ULTRA_MENU_MAX];
    InsertMenuItemW(Menu, 0, True, LoadMenuItemW(Action.Caption, Action.Enabled, Action.Checked, CreateArrangeSubMenu, ULTRA_MENU_ARRANGE));
    InsertSeparator(Menu, 0);
    InsertMenuItemW(Menu, 0, True, LoadMenuItemW(STR_VIEW, True, False, CreateSubMenuView, $FFFF));
  end else
  begin
    Action := Actions[ULTRA_MENU_PROPERTIES - ULTRA_MENU_MAX];
    InsertMenuItemA(Menu, GetMenuItemCount(Menu), True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_PROPERTIES));
    if InjectPropSeparator then
      InsertSeparator(Menu, 0);
    Action := Actions[ULTRA_MENU_PASTESHORTCUT - ULTRA_MENU_MAX];
    InsertMenuItemA(Menu, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_PASTESHORTCUT));
    Action := Actions[ULTRA_MENU_PASTE - ULTRA_MENU_MAX];
    InsertMenuItemA(Menu, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_PASTE));
    InsertSeparator(Menu, 0);
    Action := Actions[ULTRA_MENU_REFRESH - ULTRA_MENU_MAX];
    InsertMenuItemA(Menu, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, 0, ULTRA_MENU_REFRESH));
    Action := Actions[ULTRA_MENU_ARRANGE - ULTRA_MENU_MAX];
    InsertMenuItemA(Menu, 0, True, LoadMenuItemA(Action.Caption, Action.Enabled, Action.Checked, CreateArrangeSubMenu, ULTRA_MENU_ARRANGE));
    InsertSeparator(Menu, 0);
    InsertMenuItemA(Menu, 0, True, LoadMenuItemA(STR_VIEW, True, False, CreateSubMenuView, $FFFF));
  end
end;

procedure TUltraViewBkGndContextMenu.InsertSeparator(Menu: HMenu; InsertIndex: Integer);
var
  MenuInfoA: TMenuItemInfoA;
  MenuInfoW: TMenuItemInfoW;
begin
  if IsUnicode then
  begin
    FillChar(MenuInfoW, SizeOf(MenuInfoW), #0);
    MenuInfoW.cbSize := SizeOf(MenuInfoW);
    MenuInfoW.fType := MFT_SEPARATOR;
    MenuInfoW.fMask := MIIM_TYPE;
    InsertMenuItemW(Menu, InsertIndex, True, MenuInfoW)
  end else
  begin
    FillChar(MenuInfoA, SizeOf(MenuInfoA), #0);
    MenuInfoA.cbSize := SizeOf(MenuInfoA);
    MenuInfoA.fType := MFT_SEPARATOR;
    MenuInfoA.fMask := MIIM_TYPE;
    InsertMenuItemA(Menu, InsertIndex, True, MenuInfoA)
  end
end;

function TUltraViewBkGndContextMenu.LoadMenuItemA(Caption: String; Enabled, Checked: Boolean; SubMenu: HMenu; ItemID: Cardinal): TMenuItemInfoA;
begin
  FillChar(Result, SizeOf(Result), #0);
  Result.cbSize := SizeOf(Result);

  Result.fMask := MIIM_TYPE or MIIM_ID or MIIM_STATE;
  if Caption = '-' then
  begin
    Result.fType := MFT_SEPARATOR
  end else
  begin
    Result.fType := MFT_STRING;
    Result.dwTypeData := PChar(Caption);
    Result.cch := Length(Caption) + 1;
    Result.wID := ItemID;
    if not Enabled then
      Result.fState := MFS_DISABLED	or MFS_GRAYED;
    if Checked then
      Result.fState := Result.fState or MFS_CHECKED;
    if SubMenu <> 0 then
    begin
      Result.fMask := Result.fMask or MIIM_SUBMENU;
      Result.hSubMenu := SubMenu;
    end
  end
end;

function TUltraViewBkGndContextMenu.LoadMenuItemW(Caption: WideString; Enabled, Checked: Boolean; SubMenu: HMenu; ItemID: Cardinal): TMenuItemInfoW;
begin
  FillChar(Result, SizeOf(Result), #0);
  Result.cbSize := SizeOf(Result);

  Result.fMask := MIIM_TYPE or MIIM_ID or MIIM_STATE;
  if Caption = '-' then
  begin
    Result.fType := MFT_SEPARATOR
  end else
  begin
    Result.fType := MFT_STRING;
    Result.dwTypeData := PWideChar(Caption);
    Result.cch := Length(Caption) + 1;
    Result.wID := ItemID;
    if not Enabled then
      Result.fState := MFS_DISABLED	or MFS_GRAYED;
    if Checked then
      Result.fState := Result.fState or MFS_CHECKED;
    if SubMenu <> 0 then
    begin
      Result.fMask := Result.fMask or MIIM_SUBMENU;
      Result.hSubMenu := SubMenu;
    end
  end
end;

procedure TUltraViewBkGndContextMenu.LoadFromStream(S: TStream);
var
  Len: Integer;
begin
  S.Read(Len, SizeOf(Len));
  SetLength(FExcludeList, Len);
  if Len > 0 then
    S.Read(PChar(@FExcludeList[0])^, SizeOf(TGUID) * Len);
end;

procedure TUltraViewBkGndContextMenu.RemoveExcludedGUIDs(var GUIDs: TGUIDArray);
var
  i, j: Integer;
  Exclude: Boolean;
  Temp: TGUIDArray;
begin

  for i := 0 to Length(GUIDs) - 1 do
  begin
    j := 0;
    Exclude := False;
    while not Exclude and (j < Length(ExcludeList)) do
    begin
      if IsEqualGUID(ExcludeList[j], GUIDs[i]) then
        Exclude := True;
      Inc(j)
    end;
    if not Exclude then
    begin
      SetLength(Temp, Length(Temp) + 1);
      Temp[Length(Temp) - 1] := GUIDs[i]
    end
  end;
  GUIDs := Temp;
end;

procedure TUltraViewBkGndContextMenu.SaveToStream(S: TStream);
var
  Len: Integer;
begin
  Len := Length(ExcludeList);
  S.Write(Len, SizeOf(Len));
  if Len > 0 then
    S.Write(PChar(@FExcludeList[0])^, SizeOf(TGUID) * Len);
end;

procedure TUltraViewBkGndContextMenu.SetActions(Index: Integer; Value: TTntAction);
begin
  FActions[Index] := Value
end;

procedure TUltraViewBkGndContextMenu.SetItems(Index: Integer; Value: PUltraContextMenuMap);
begin
  PUltraContextMenuMap(ItemList[Index]).Menu := nil;
  Dispose(PUltraContextMenuMap( ItemList[Index]));
  ItemList[Index] := Value
end;

procedure TUltraViewBkGndContextMenu.Show(Browser: IShellBrowser;
  ScreenPt: TPoint; Window: HWnd; NS: TNamespace; var IsNewMenuCommand: Boolean);
const
  MAX_VERB_LEN = 128;
var
  GUIDs: TGUIDArray;
  i, j, idCmdFirst, idCmdLast, idCmdNext, NewMenuFirst, NewMenuLast: Integer;
  Unk: IUnknown;
  ContextMenu, CurrentContextMenu: IContextMenu;
  Menu: HMenu;
  MenuCmd: Integer;
  Rslt: HResult;
  PIDL: PItemIDList;
  MenuInfoA: TMenuItemInfoA;
  MenuInfoW: TMenuItemInfoW;
  ContextMenu2: IContextMenu2;
  ContextMenu3: IContextMenu3;
  Flags: LongWord;
  VerbA: string;
  VerbW: WideString;
  GenericVerb: Pointer;
  InvokeInfo: TCMInvokeCommandInfoEx;
  Map: PUltraContextMenuMap;
  DataObj: IDataObject;
  ShellIDList: TCommonShellIDList;
  PIDLList: TCommonPIDLList;
  HDrop: TCommonHDrop;
  FileList: TTntStringList;
  {$IFDEF DEGUG_BKGND_MENU}
  F: TStringList;
  {$ENDIF}
  ObjWithSite: IObjectWithSite;
begin
  {$IFDEF DEGUG_BKGND_MENU}
  F := TStringList.Create;
  F.Add('Entering Bkgnd Menu call');
  {$ENDIF}

  IsNewMenuCommand := False;
  NewMenuFirst := -1;
  NewMenuLast := -1;

  if Assigned(NS) then
  begin
    {$IFDEF DEGUG_BKGND_MENU}
    F.Add('Creating PopupMenu');
    {$ENDIF}
    Menu := CreatePopupMenu;
    {$IFDEF DEGUG_BKGND_MENU}
    F.Add('Menu Created');
    {$ENDIF}
    try
      {$IFDEF DEGUG_BKGND_MENU}
      F.Add('Searching for BkGnd Menu GUIDs');
      {$ENDIF}
      FindBkGndContextMenus(GUIDs);
      {$IFDEF DEGUG_BKGND_MENU}
      F.Add('Found GUIDs...');
      for i := 0 to Length(GUIDs) - 1 do
        F.Add('GUID = ' + GUIDToString(GUIDs[i]));
      {$ENDIF}

      if (Length(GUIDs) > 0) and NS.FileSystem then
      begin
        idCmdFirst := 1000;
        idCmdLast := ULTRA_MENU_MAX;
        DataObj := TCommonDataObject.Create;
        // The "New" extension uses the PIDL
        PIDL := PIDLMgr.CopyPIDL(NS.AbsolutePIDL);

        {$IFDEF DEGUG_BKGND_MENU}
        F.Add('Creating an IDataObject');
        {$ENDIF}
        // Create a valid IDataObject
        ShellIDList := TCommonShellIDList.Create;
        PIDLList := TCommonPIDLList.Create;
        try
          PIDLList.SharePIDLs := False;
          PIDLList.Add(PIDLMgr.StripLastID(PIDLMgr.CopyPIDL(NS.AbsolutePIDL)));
          PIDLList.CopyAdd(NS.RelativePIDL);
          ShellIDList.AssignPIDLs(PIDLList);
          ShellIDList.SaveToDataObject(DataObj)
        finally
          PIDLList.Free;
          ShellIDList.Free
        end;
        FileList := TTntStringList.Create;
        HDrop := TCommonHDrop.Create;
        try
          FileList.Add(NS.NameForParsing);
          HDrop.AssignFilesW(FileList);
          HDrop.SaveToDataObject(DataObj)
        finally
          FileList.Free;
          HDrop.Free
        end;
        {$IFDEF DEGUG_BKGND_MENU}
        F.Add('IDataObject created successfully');
        {$ENDIF}

        {$IFDEF DEGUG_BKGND_MENU}
        F.Add('Creating the IContextMenu object');
        {$ENDIF}
        for i := 0 to Length(GUIDs) - 1 do
        begin
          if not IsEqualGUID(GUIDs[i], GUID_NULL) then
          begin
            {$IFDEF DEGUG_BKGND_MENU}
             F.Add('Building the Shell Context Menu Items');
            {$ENDIF}
            // We will do the New Menu ourselves
            if Succeeded( coCreateInstance(GUIDs[i], nil, CLSCTX_INPROC_SERVER, IContextMenu, Unk)) then
            begin
              {$IFDEF DEGUG_BKGND_MENU}
               F.Add('Item ' + IntToStr(i) + ' of ' + IntToStr(Length(GUIDs)) + ' coCreateInstance succeeded');
               F.Add('Returned an IUnknown of ' + IntToStr( Integer(Unk)));
               F.Add('Querying for iContextMenu');
              {$ENDIF}

              ContextMenu := Unk as IContextMenu;

              {$IFDEF DEGUG_BKGND_MENU}
              if Assigned(ContextMenu) then
                F.Add('ContextMenu interface succeeded')
              else
                F.Add('ContextMenu interface did not succeed');
                F.Add('Initalizing on IShellExtInit');
              {$ENDIF}

              if Succeeded((Unk as IShellExtInit).Initialize(PIDL, DataObj, 0)) then
              begin
                Flags := CMF_NORMAL or CMF_EXPLORE;
                if GetKeyState(VK_SHIFT) and $8000 <> 0 then
                  Flags := Flags or CMF_EXTENDEDVERBS;

                {$IFDEF DEGUG_BKGND_MENU}
                F.Add('IShellExtInit.Initialize succeeded');
                F.Add('IShellExtInit.Initialize succeeded');
                {$ENDIF}

                if ContextMenu.QueryInterface(IContextMenu3, Pointer(ContextMenu3)) = E_NOINTERFACE then
                  ContextMenu3 := nil;
                if ContextMenu.QueryInterface(IContextMenu2, Pointer(ContextMenu2)) = E_NOINTERFACE then
                  ContextMenu2 := nil;

                if Assigned(ContextMenu3) then
                  CurrentContextMenu := ContextMenu3
                else
                if Assigned(ContextMenu2) then
                  CurrentContextMenu := ContextMenu2
                else
                  CurrentContextMenu := ContextMenu;

                if CurrentContextMenu.QueryInterface(IObjectWithSite, ObjWithSite) = NOERROR then
                  ObjWithSite.SetSite(Browser);

                Rslt := CurrentContextMenu.QueryContextMenu(Menu, 0, idCmdFirst, idCmdLast, Flags);

                if Succeeded(Rslt) then
                begin
                  idCmdNext := HResultCode(Rslt);

                  if IsEqualCLSID(CLSID_NewMenu, GUIDs[i]) then
                  begin
                    NewMenuFirst := idCmdFirst;
                    NewMenuLast := idCmdFirst + idCmdNext - 1;
                  end;

                  if idCmdNext < 1000 then
                    idCmdNext := idCmdFirst + idCmdNext;

                  // Map the items added to the current IContextMenu2 instance
                  for j := idCmdFirst to idCmdFirst + idCmdNext - 2 do
                  begin
                    if IsUnicode then
                    begin
                      FillChar(MenuInfoW, SizeOf(MenuInfoW), #0);
                      MenuInfoW.cbSize := SizeOf(MenuInfoW);
                      MenuInfoW.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU	or MIIM_STATE	or MIIM_CHECKMARKS or MIIM_DATA;
                      if GetMenuItemInfoW(Menu, j, False, MenuInfoW) then
                        Add(CurrentContextMenu, MenuInfoW.wID, MenuInfoW.hSubMenu, idCmdFirst);
                    end else
                    begin
                      FillChar(MenuInfoA, SizeOf(MenuInfoA), #0);
                      MenuInfoA.cbSize := SizeOf(MenuInfoA);
                      MenuInfoA.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU	or MIIM_STATE	or MIIM_CHECKMARKS or MIIM_DATA;
                      if GetMenuItemInfoA(Menu, j, False, MenuInfoA) then
                        Add(CurrentContextMenu, MenuInfoA.wID, MenuInfoA.hSubMenu, idCmdFirst);
                    end
                  end;
                  idCmdFirst := idCmdNext
                end
              end
            end;
            ValidateMenuSeparators(Menu);
          end
        end;
        PIDLMgr.FreePIDL(PIDL);
        InjectStandardItems(Menu, False)
      end else
        InjectStandardItems(Menu, True);

      MenuCmd := Cardinal( TrackPopupMenuEx(
                     Menu,
                     TPM_LEFTALIGN or TPM_RETURNCMD or TPM_RIGHTBUTTON,
                     ScreenPt.X, ScreenPt.Y, Window, nil));

      if MenuCmd > 0 then
      begin
        // See if it is a new command
        IsNewMenuCommand := (MenuCmd >= NewMenuFirst) and (MenuCmd <= NewMenuLast);
        if MenuCmd > ULTRA_MENU_MAX then
        begin
          // It is an internal command
          MenuCmd := MenuCmd - ULTRA_MENU_MAX;
          Actions[MenuCmd].Execute
        end else
        begin
          // It is a context menu extension command

          // First find the verb associated with the command
          Map := FindMenuMapByID(MenuCmd);
          if Assigned(Map) then
          begin
            CurrentContextMenu := Map^.Menu;
          
            MenuCmd := MenuCmd - Map^.Offset;
            if IsUnicode then
            begin
              SetLength(VerbW, MAX_VERB_LEN);
              GenericVerb := @VerbW[1];
              Flags := GCS_VERBW
            end else
            begin
              SetLength(VerbA, MAX_VERB_LEN);
              GenericVerb := @VerbA[1];
              Flags := GCS_VERBA
            end;
            if Succeeded(CurrentContextMenu.GetCommandString(MenuCmd, Flags, nil, GenericVerb, MAX_VERB_LEN)) then
            begin
              if IsUnicode then
              begin
                SetLength(VerbW, lstrlenW(PWideChar( VerbW)));
              end else
              begin
                SetLength(VerbA, lstrlen(PChar( VerbA)));
                VerbW := VerbA
              end;
            end;

            FillChar(InvokeInfo, SizeOf(InvokeInfo), #0);
            with InvokeInfo do
            begin
              { For some reason the lpVerbW won't work }
              lpVerb := MakeIntResourceA(MenuCmd);
              if IsUnicode then
              begin
                fMask := CMIC_MASK_UNICODE;
                lpVerbW := MakeIntResourceW(MenuCmd)
              end;
              // Win95 get confused if size = TCMInvokeCommandInfoEx
              if IsUnicode then
                cbSize := SizeOf(TCMInvokeCommandInfoEx)
              else
                cbSize := SizeOf(TCMInvokeCommandInfo);

              hWnd := Window;
              nShow := SW_SHOWNORMAL;
            end;
            CurrentContextMenu.InvokeCommand(InvokeInfo)
          end
        end
      end
    finally
      if Assigned(ObjWithSite) then
        ObjWithSite.SetSite(nil);
      Clear;
      DestroyMenu(menu);
      {$IFDEF DEGUG_BKGND_MENU}
      F.SaveToFile(AppFilesPath + '\ContextMenuDebug.txt');
      F.Free;
      {$ENDIF}

    end
  end
end;

procedure TUltraViewBkGndContextMenu.StoreMenuIDs(Menu: HMenu);
var
  i: Integer;
  MenuInfoA: TMenuItemInfoA;
  MenuInfoW: TMenuItemInfoW;
begin
  MenuIDStore.Clear;
  for i := 0 to GetMenuItemCount(Menu) - 1 do
  begin
    if IsUnicode then
    begin
      FillChar(MenuInfoW, SizeOf(MenuInfoW), #0);
      MenuInfoW.cbSize := SizeOf(MenuInfoW);
      MenuInfoW.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU	or MIIM_STATE	or MIIM_CHECKMARKS or MIIM_DATA;
      if GetMenuItemInfoW(Menu, i, True, MenuInfoW) then
        MenuIDStore.Add(Pointer( MenuInfoW.wID))
    end else
    begin
      FillChar(MenuInfoA, SizeOf(MenuInfoA), #0);
      MenuInfoA.cbSize := SizeOf(MenuInfoA);
      MenuInfoW.fMask := MIIM_ID or MIIM_TYPE or MIIM_SUBMENU	or MIIM_STATE	or MIIM_CHECKMARKS or MIIM_DATA;
      if GetMenuItemInfoA(Menu, i, True, MenuInfoA) then
        MenuIDStore.Add(Pointer( MenuInfoA.wID))
    end
  end
end;

procedure TUltraViewBkGndContextMenu.ValidateMenuSeparators(Menu: HMenu);
var
  i: Integer;
  Separators: TList;
begin
  Separators := TList.Create;
  try
    FillSeparatorList(Menu, Separators);
     // Walk down the list and remove any duplicated separators
    for i := Separators.Count - 1 downto 1 do
    begin
      if (Integer(Separators[i]) <> -1) and (Integer(Separators[i-1]) <> -1) then
        DeleteMenu(Menu, i, MF_BYPOSITION)
    end;
    FillSeparatorList(Menu, Separators);
    if Separators.Count > 0 then
    begin
      if Integer( Separators[Separators.Count - 1]) = -1 then
        InsertSeparator(Menu, Separators.Count);
      if Integer( Separators[0]) = -1 then
        InsertSeparator(Menu, 0);
    end
  finally
    Separators.Free
  end
end;

{ TNamespaceGrouping }
constructor TNamespaceGrouping.Create;
begin
  inherited Create;
  FGroups := TList.Create;
end;

destructor TNamespaceGrouping.Destroy;
begin
  Clear;
  FreeAndNil(FGroups);
  inherited Destroy;
end;

function TNamespaceGrouping.AddGroup: TVirtualNamespaceList;
begin
  Result :=  TVirtualNamespaceList.Create;
  FGroups.Add(Result)
end;

function TNamespaceGrouping.GetCount: Integer;
begin
  Result := FGroups.Count
end;

function TNamespaceGrouping.GetGroups(Index: Integer): TVirtualNameSpaceList;
begin
  Result := TVirtualNameSpaceList( FGroups[Index])
end;

function TNamespaceGrouping.GroupToArray(Group: TVirtualNamespaceList): TNamespaceArray;
var
  i: Integer;
begin
  SetLength(Result, Group.Count);
  for i := 0 to Group.Count - 1 do
    Result[i] := Group[i]
end;

procedure TNamespaceGrouping.Clear;
var
  i: Integer;
begin
  for i := 0 to FGroups.Count -1 do
    TObject( FGroups[i]).Free
end;

procedure TNamespaceGrouping.SetGroups(Index: Integer; Value: TVirtualNameSpaceList);
begin
  FGroups[Index] := Value
end;

procedure TNamespaceGrouping.SortIntoGroup(NS: TNamespace);
var
  Done: Boolean;
  i: Integer;
begin
  Done := False;
  i := 0;
  while not Done and (i < Count) do
  begin
    Done := Groups[i][0].Parent.ComparePIDL(NS.Parent.AbsolutePIDL, True, 0) = 0;
    if Done then
      Groups[i].Add(NS);
    Inc(i)
  end;
  if not Done then
    AddGroup.Add(NS)
end;

{ TUltraSpTBXTabItem }
constructor TUltraSpTBXTabItem.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Selected := TMemoryStream.Create;
  History := TMemoryStream.Create;
  MRU := TMemoryStream.Create;
end;

destructor TUltraSpTBXTabItem.Destroy;
var
  InPlaceNotify: IUltraInplaceNotifications;
begin
  if Assigned(InPlaceWindow) then
  begin
    InPlaceWindow.SaveConfig;
    InPlaceWindow.RegisterNotifier(nil);
    if Supports(InPlaceWindow, IUltraInplaceNotifications, InPlaceNotify) then
      InPlaceNotify.TabChanging(ntsDestroying);
    InPlaceWindow.ShowWindow(False);
    InPlaceWindow.SetParentWnd(nil);
    InPlaceWindow.DestroyWindow;
    InPlaceWindow := nil;
  end;
  PIDLMgr.FreePIDL(PIDL);
  FreeAndNil(FSelected);
  FreeAndNil(FHistory);
  FreeAndNil(FMRU);
  inherited Destroy;
end;

procedure TUltraSpTBXTabItem.SetInPlaceWindow(const Value: IUltraInPlaceWindow);
begin
  if Value <> FInPlaceWindow then
  begin
    FInplaceWindow := Value;
    FInPlaceTab := True
  end
end;

{ TUltraExplorerEasyListview }
constructor TUltraExplorerEasyListview.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CacheDoubleBufferBits := True;
  if AOwner is TUltraColumnModeView then
  begin
    ChangeNotifierEnabled := TUltraColumnModeView( AOwner).ChangeNotifierEnabled;
    EnableGesture := TUltraColumnModeView( AOwner).EnableGesture;
  end;
end;

destructor TUltraExplorerEasyListview.Destroy;
var
   LVMode: TVirtualColumnModeView;
begin
  if not (Parent is TCustomVirtualColumnModeView) and Assigned(ColumnModeView) then
  begin 
    LVMode := ColumnModeView;
    ColumnModeView := nil;
    LVMode.Free;
  end;
  inherited Destroy;
end;

function TUltraExplorerEasyListview.AddCustomItemEx(Group: TEasyGroup; NS: TNamespace): TExplorerItem;
begin
  Result := nil;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := ColumnModeView.ActiveView.AddCustomItem(Group, NS, True)
  end else
    Result := AddCustomItem(Group, NS, True)
end;

function TUltraExplorerEasyListview.BrowseObject(pidl: PItemIDList;
  flags: UINT): HResult;
begin
  BrowseToByPIDL(pidl, True);
  Result := S_OK
end;

function TUltraExplorerEasyListview.ContextSensitiveHelp(fEnterMode: BOOL): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.DoItemCompare(Column: TEasyColumn; Group: TEasyGroup; Item1: TEasyItem; Item2: TEasyItem): Integer;
var
  ColumnIndex: Integer;
  IsFolder1, IsFolder2: Boolean;
begin
  ColumnIndex := Column.Index;
  if (ColumnIndex = ID_SIZE_COLUMN) then
  begin
    IsFolder1 := TExplorerItem(Item1).Namespace.Folder and not TExplorerItem(Item1).Namespace.Browsable;
    IsFolder2 := TExplorerItem(Item2).Namespace.Folder and not TExplorerItem(Item2).Namespace.Browsable;

    Result := 0;
    if SortFolderFirstAlways then
    begin
      if IsFolder1 xor IsFolder2 then
      begin
        if IsFolder1 then
          Result := -1
        else
          Result := 1;
      end
    end;

    if Result = 0 then
    begin
      if IsFolder1 and IsFolder2 then
      begin
        if TUltraExplorerItem( Item2).FolderSize = TUltraExplorerItem( Item1).FolderSize then
          Result := 0
        else
        if TUltraExplorerItem( Item2).FolderSize > TUltraExplorerItem( Item1).FolderSize then
          Result := -1
        else
          Result := 1
      end else
      begin
        if TUltraExplorerItem( Item2).Namespace.SizeOfFileInt64 = TUltraExplorerItem( Item1).Namespace.SizeOfFileInt64 then
          Result := 0
        else
        if TUltraExplorerItem( Item2).Namespace.SizeOfFileInt64 >  TUltraExplorerItem( Item1).Namespace.SizeOfFileInt64 then
          Result := -1
        else
          Result := 1
      end;
      // Secondary level of sorting is on the Name
      if (Result = 0) and (ColumnIndex > 0) then
        Result := TExplorerItem(Item2).Namespace.ComparePIDL(TExplorerItem(Item1).Namespace.RelativePIDL, False, 0);
      if (ColumnIndex > -1) and (Column.SortDirection = esdDescending) then
        Result := -Result
    end
  end else
  Result := inherited DoItemCompare(Column, Group, Item1, Item2);
end;

function TUltraExplorerEasyListview.EnableModelessSB(Enable: BOOL): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.FindItemByPathEx(Path: WideString): TExplorerItem;
begin
  Result := nil;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := TExplorerItem( ColumnModeView.ActiveView.FindItemByPath(Path))
  end else
    Result := TExplorerItem( FindItemByPath(Path))
end;

function TUltraExplorerEasyListview.FirstSelection: TExplorerItem;
begin
  Result := nil;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := TExplorerItem( ColumnModeView.ActiveView.Selection.First)
  end else
    Result := TExplorerItem( Selection.First)
end;

function TUltraExplorerEasyListview.GetColumnModeView: TUltraColumnModeView;
begin
  Result := inherited ColumnModeView as TUltraColumnModeView
end;

function TUltraExplorerEasyListview.GetControlWindow(ID: UINT; out Wnd: HWND): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.GetFileObjects: TFileObjects;
begin
  Result := inherited FileObjects
end;

function TUltraExplorerEasyListview.GetShowInactiveEx: Boolean;
begin
  if Assigned(ColumnModeView) then
    Result := ColumnModeView.ShowInactive
  else
    Result := ShowInactive
end;

function TUltraExplorerEasyListview.GetViewStateStream(Mode: DWORD;
  out Stream: IStream): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.GetWindow(out wnd: HWnd): HResult;
begin
  if HandleAllocated then
  begin
    wnd := Handle;
    Result := S_OK
  end else
    Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.InsertMenusSB(hMenuShared: HMENU;
  out MenuWidths: TOleMenuGroupWidths): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.ItemCountEx: Integer;
begin
  Result := 0;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := ColumnModeView.ActiveView.ItemCount
  end else
    Result := ItemCount
end;

function TUltraExplorerEasyListview.NextSelection(Item: TExplorerItem): TExplorerItem;
begin
  Result := nil;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := TExplorerItem( ColumnModeView.ActiveView.Selection.Next(Item))
  end else
    Result := TExplorerItem( Selection.Next(Item))
end;

function TUltraExplorerEasyListview.OnViewWindowActive(
  var ShellView: IShellView): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.QueryActiveShellView(
  var ShellView: IShellView): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.RemoveMenusSB(hMenuShared: HMENU): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.SelectedToNamespaceArrayEx: TNamespaceArray;
begin
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := ColumnModeView.ActiveView.SelectedToNamespaceArray
  end else
    Result := SelectedToNamespaceArray
end;

function TUltraExplorerEasyListview.SelectionCount: Integer;
begin
  Result := 0;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := ColumnModeView.ActiveView.Selection.Count
  end else
    Result := Selection.Count
end;

function TUltraExplorerEasyListview.SendControlMsg(ID, Msg: UINT;
  wParm: WPARAM; lParm: LPARAM; var Rslt: LResult): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.SetMenuSB(hMenuShared: HMENU;
  hOleMenuReserved: HOLEMENU; hwndActiveObject: HWND): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.SetStatusTextSB(StatusText: POleStr): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.SetToolbarItems(TBButton: PTBButton;
  nButtons, uFlags: UINT): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.TranslateAcceleratorSB(Msg: PMsg; ID: Word): HResult;
begin
  Result := E_NOTIMPL
end;

function TUltraExplorerEasyListview.ValidRootNamespaceEx: Boolean;
begin
  Result := False;
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      Result := ColumnModeView.ActiveView.ValidRootNamespace
  end else
    Result := ValidRootNamespace
end;

procedure TUltraExplorerEasyListview.BrowseToByCallback(PIDL: PItemIDList;
  NewTab: Boolean);
var
  Browser: IUltraBrowser;
begin
// JDK  if Application.MainForm.GetInterface(IUltraBrowser, Browser) then
 //   Browser.BrowseTo(PIDL, True, NewTab, True)
end;

procedure TUltraExplorerEasyListview.BrowseToPrevLevelEx;
begin
  if Assigned(ColumnModeView) then
    ColumnModeView.BrowseToPrevLevel
  else
    BrowseToPrevLevel
end;

procedure TUltraExplorerEasyListview.ChangeLinkChanging(Server: TObject; NewPIDL: PItemIDList);
begin
  //
  // If the Listview is a child of a ColumnModeView the ColumnModeView will handle the
  // change link
  //
  if not Assigned(ColumnModeView) then
    inherited ChangeLinkChanging(Server, NewPIDL);
end;

procedure TUltraExplorerEasyListview.ChangeLinkDispatch;
begin
  if not LockChangeLink then
    inherited ChangeLinkDispatch;
end;

procedure TUltraExplorerEasyListview.CheckFocus;
begin
  if not DraggingFromBackground and HandleAllocated and Assigned(Parent) then
    inherited CheckFocus;
end;

procedure TUltraExplorerEasyListview.CMRecreateWnd(var Msg: TMessage);
var
  WasNotified: Boolean;
begin
  WasNotified := ChangeNotifierEnabled;
  inherited;
  ChangeNotifierEnabled := WasNotified;
end;

procedure TUltraExplorerEasyListview.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
end;

procedure TUltraExplorerEasyListview.CutToClipboard;
begin
  if Assigned(OpenViewer) then
    OpenViewer('');
  inherited CutToClipboard;
end;

procedure TUltraExplorerEasyListview.DoEnumFinished;
begin
  inherited DoEnumFinished;
  FolderSizeLoadThread(False, False);
end;

procedure TUltraExplorerEasyListview.DoExplorerItemClass(var ItemClass: TExplorerItemClass);
begin
  ItemClass := TUltraExplorerItem;
  if Assigned(OnExplorerItemClass) then
    OnExplorerItemClass(Self, ItemClass);
end;

procedure TUltraExplorerEasyListview.DoItemEditBegin(Item: TEasyItem; var Column: Integer; var Allow: Boolean);
begin
  if not DraggingFromBackground then
    inherited DoItemEditBegin(Item, Column, Allow)
  else
    Allow := False
end;

procedure TUltraExplorerEasyListview.DoItemGetCaption(Item: TEasyItem; Column: Integer; var ACaption: WideString);
var
  UltraItem: TUltraExplorerItem;
begin
  UltraItem := TUltraExplorerItem( Item);
  if {Attribs.Listview.FolderSizeShow and} (Column = ID_SIZE_COLUMN) and UltraItem.Namespace.Folder and not UltraItem.Namespace.Browsable and UltraItem.Namespace.Parent.FileSystem then
  begin
    if UltraItem.Namespace.FileSystem and UltraItem.Namespace.Folder and not UltraItem.Namespace.Browsable then
    begin
      if (uisThreadLoaded in TUltraExplorerItem( Item).UltraStates) then
      begin
        case Attribs.Listview.FolderSizeDisplay of
          bsKiloBytes:
            begin
              ACaption := Format('%0.0n '+ 'KB', [UltraItem.FolderSize / 1024]);
              if (ACaption = '0 KB') and (UltraItem.FolderSize > 0) then
                ACaption := '1 KB';
              ACaption := ACaption + '  '
            end;
          bsMegaBytes:
            begin
              ACaption := Format('%0.0n '+ 'MB', [UltraItem.FolderSize / 1048576]);
              if (ACaption = '0 MB') and (UltraItem.FolderSize > 0) then
                ACaption := '1 MB';
              ACaption := ACaption + '  '
            end;
          bsGigiBytes:
            begin
              ACaption := Format('%0.1n '+ 'GB', [UltraItem.FolderSize / 1073741824]);
              if (ACaption = '0 GB') and (UltraItem.FolderSize > 0) then
                ACaption := '1 GB';
              ACaption := ACaption + '  '
            end;
          bsTereBytes:
            begin
              ACaption := Format('%0.1n '+ 'TB', [UltraItem.FolderSize / 1099511627776]);
              if (ACaption = '0 TB') and (UltraItem.FolderSize > 0) then
                ACaption := '1 TB';
              ACaption := ACaption + '  '
            end
        end;
      end else
      begin
        if (uisThreadLoading in TUltraExplorerItem( Item).UltraStates) then
          ACaption := STR_LOADING + '  '
      end
    end
  end else
    inherited DoItemGetCaption(Item, Column, ACaption);
end;

procedure TUltraExplorerEasyListview.DoItemSelectionsChanged;
begin
  inherited DoItemSelectionsChanged;
end;

procedure TUltraExplorerEasyListview.DoItemSetCaption(Item: TEasyItem; Column: Integer; const Caption: WideString);
var
  WasOpen, WasCmdLine: Boolean;
begin
  WasOpen := False;
  WasCmdLine := False;
  if (Assigned(ViewerOpenFile)) and Assigned(OpenViewer) then
  begin
    if WideUpperCase(ViewerOpenFile) = WideUpperCase(TExplorerItem(Item).Namespace.NameForParsing) then
    begin
      WasOpen := True;
      OpenViewer('');
    end
  end;
  if Assigned(CmdLineSetPath) then
  begin
    if TExplorerItem(Item).Namespace.ComparePIDL(CmdLineCurrentPath^, True) = 0 then
    begin
      CmdLineSetPath(TExplorerItem(Item).Namespace.Parent);
      WasCmdLine := True
    end
  end;

  inherited DoItemSetCaption(Item, Column, Caption);
  if WasOpen and Assigned(OpenViewer) then
    OpenViewer(TExplorerItem(Item).Namespace.NameForParsing);
  if WasCmdLine and Assigned(CmdLineSetPath) then
    CmdLineSetPath(TExplorerItem(Item).Namespace)
end;

procedure TUltraExplorerEasyListview.DoItemStructureChange;
begin
  inherited DoItemStructureChange;
  QuickFilterUpdatedNeeded := True
end;

procedure TUltraExplorerEasyListview.DoRootChange;
begin
  inherited DoRootChange;
  if Assigned(RootFolderNamespace) and ChangeNotifierEnabled then
    ChangeNotifier.NotifyWatchFolder(Self, RootFolderNamespace.NameForParsing);
end;

procedure TUltraExplorerEasyListview.DoRootChanging(const NewRoot: TRootFolder; Namespace: TNamespace; var Allow: Boolean);
begin
  inherited DoRootChanging(NewRoot, Namespace, Allow);
  SumFolderAbort := True
end;

procedure TUltraExplorerEasyListview.DoThreadCallback(var Msg: TWMThreadRequest);
var
  NS: TNamespace;
  AnItem: TUltraExplorerItem;
  FolderSizeRequest: TUltraFolderSizeThreadRequest;
begin
  AnItem := TUltraExplorerItem( Msg.Request.Item);
  case Msg.RequestID of
    TID_FOLDERSIZE:
      begin
        if ValidateNamespace(AnItem, NS) then
        begin
          FolderSizeRequest := Msg.Request as TUltraFolderSizeThreadRequest;
          if FolderSizeRequest.FolderSize > FolderSizeMax then
            FolderSizeMax := FolderSizeRequest.FolderSize;  
          AnItem.FolderSize := FolderSizeRequest.FolderSize;
          AnItem.UltraStates := AnItem.UltraStates + [uisThreadLoaded] - [uisThreadLoading];
          Groups.InvalidateItem(AnItem, False);
        end
      end
  end;
  // STUPID this call releases the Request!!!!!!!  MUST DO IT LAST
  inherited DoThreadCallback(Msg);
end;

procedure TUltraExplorerEasyListview.EndUpdate(Invalidate: Boolean = True);
begin
  if QuickFiltered then
  begin
    if HandleAllocated then
      SendMessage(Handle, WM_SETREDRAW, 0, 0)
  end;

  inherited EndUpdate(Invalidate);
  if QuickFiltered then
  begin
    if QuickFilterUpdatedNeeded then
    begin
      QuickFilter;
      QuickFilterUpdatedNeeded := False;
    end;
    if HandleAllocated then
    begin
      SendMessage(Handle, WM_SETREDRAW, 1, 0);
      InvalidateRect(Handle, nil, False);
    end
  end
end;

procedure TUltraExplorerEasyListview.FolderSizeClear;
var
  Item: TUltraExplorerItem;
begin
  Item := TUltraExplorerItem(Groups.FirstItem);
  while Assigned(Item) do
  begin
    Item.FolderSize := -1;
    Item := TUltraExplorerItem(Groups.NextItem(Item))
  end;
  FFolderSizeMax := -1;
end;

procedure TUltraExplorerEasyListview.FolderSizeLoadThread(SelectedOnly, Force: Boolean);
var
  ThreadRequest: TUltraFolderSizeThreadRequest;
  UltraItem: TUltraExplorerItem;
  Allow: Boolean;
begin
  if Attribs.Listview.FolderSizeShow or Force then
  begin
    UltraItem := TUltraExplorerItem(Groups.FirstItem);
    while Assigned(UltraItem) do
    begin
      if UltraItem.Namespace.Folder and not UltraItem.Namespace.Browsable and UltraItem.Namespace.Parent.FileSystem then
      begin
        Allow := True;
        if SelectedOnly then
          Allow := UltraItem.Selected;
        if Allow then
        begin
          ThreadRequest := TUltraFolderSizeThreadRequest.Create;
          ThreadRequest.PIDL := PIDLMgr.CopyPIDL(UltraItem.Namespace.AbsolutePIDL);
          ThreadRequest.Window := Self;
          ThreadRequest.Item := UltraItem;
          ThreadRequest.Priority := 100;
          ThreadRequest.ID := TID_FOLDERSIZE;
          TUltraExplorerItem(UltraItem).UltraStates := UltraItem.UltraStates - [uisThreadLoaded] + [uisThreadLoading];
          GlobalThreadManager.AddRequest(ThreadRequest, True);
        end
      end;
      UltraItem := TUltraExplorerItem(Groups.NextItem(UltraItem));
    end;
    FolderSizeMax := 0;
  end;
end;

procedure TUltraExplorerEasyListview.HandleMouseUp(Button: TCommonMouseButton; Msg: TWMMouse);
var
  ScreenRect: TRect;
  ScreenPt: TPoint;
begin
  inherited HandleMouseUp(Button, Msg);
  // If mouse up was in the window then it was not a drag from the background.
  if DraggingFromBackground then
  begin
    DraggingFromBackground := False;
    // Only set focus if mouse is still over our window
    if Assigned(ColumnModeView) then
    begin
      ScreenRect.TopLeft := ColumnModeView.ClientToScreen(ColumnModeView.ClientRect.TopLeft);
      ScreenRect.BottomRight := ColumnModeView.ClientToScreen(ColumnModeView.ClientRect.BottomRight);
      ScreenPt := ClientToScreen(SmallPointToPoint(Msg.Pos));
      if PtInRect(ScreenRect, ScreenPt) then
        CheckFocus
    end;
    if WindowFromPoint(Mouse.CursorPos) = Handle then
      CheckFocus
  end;
end;

procedure TUltraExplorerEasyListview.RebuildEx;
begin
  if Assigned(ColumnModeView) then
    ColumnModeView.Rebuild
  else
    Rebuild
end;

procedure TUltraExplorerEasyListview.RefreshEx;
begin
  if Assigned(ColumnModeView) then
    ColumnModeView.Refresh
  else
    RereadAndRefresh(True)
end;

procedure TUltraExplorerEasyListview.SelectedFilesDelete;
begin
  if Assigned(OpenViewer) then
    OpenViewer('');
  inherited SelectedFilesDelete;
end;

procedure TUltraExplorerEasyListview.SelectionClearAll;
begin
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      ColumnModeView.ActiveView.Selection.ClearAll
  end else
    Selection.ClearAll
end;

procedure TUltraExplorerEasyListview.SelectionInvert;
begin
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      ColumnModeView.ActiveView.Selection.Invert
  end else
    Selection.Invert
end;

procedure TUltraExplorerEasyListview.SelectionSelectAll;
begin
  if Assigned(ColumnModeView) then
  begin
    if Assigned(ColumnModeView.ActiveView) then
      ColumnModeView.ActiveView.Selection.SelectAll
  end else
    Selection.SelectAll
end;

procedure TUltraExplorerEasyListview.SetEnableGesture(const Value: Boolean);
begin
  if FEnableGesture <> Value then
  begin
    FEnableGesture := Value;
    if Value then
    begin
      Gesture.Enabled := True;
      Selection.MouseButtonSelRect := Selection.MouseButtonSelRect - [cmbRight];
      if Assigned(ColumnModeView) then
        ColumnModeView.EnableGesture := Value
    end else
    begin
      Gesture.Enabled := False;
      Selection.MouseButtonSelRect := Selection.MouseButtonSelRect + [cmbRight];
      if Assigned(ColumnModeView) then
        ColumnModeView.EnableGesture := Value
    end
  end
end;

procedure TUltraExplorerEasyListview.SetFileObjects(const Value: TFileObjects);
begin
  inherited FileObjects := Value;
  if Assigned(ColumnModeView) then
    ColumnModeView.FileObjects := Value
end;

procedure TUltraExplorerEasyListview.SetFocusEx;
var
  TargetView: TColumnModeEasyListview;
begin
  if Assigned(ColumnModeView) then
  begin
    TargetView := ColumnModeView.ActiveView;
    if not Assigned(TargetView) then
      TargetView := ColumnModeView.LastActiveView;
    if not Assigned(TargetView) and (ColumnModeView.ViewCount > 0) then
      TargetView := ColumnModeView.Views[0];
    if Assigned(TargetView) then
    begin
      if (GetFocus <> TargetView.Handle) then
      begin
        if (TargetView.Selection.Count = 0) and (TargetView.ItemCount > 0) then
          TargetView.Items[0].Selected := True;
        TargetView.SetFocus
      end
    end
  end else
    if CanFocus then
      SetFocus
end;

procedure TUltraExplorerEasyListview.SetColumnModeView(const Value: TUltraColumnModeView);
begin
  inherited ColumnModeView := Value
end;

procedure TUltraExplorerEasyListview.SetFolderSizeMax(const Value: Int64);
begin
  if Value <> FFolderSizeMax then
  begin
    FFolderSizeMax := Value;
    SafeInvalidateRect(nil, False)
  end
end;

procedure TUltraExplorerEasyListview.SetChangeNotifierEnabled(
  const Value: boolean);
begin
  if FChangeNotifierEnabled <> Value then
  begin
    FChangeNotifierEnabled := Value;
    if Value then
    begin
      ChangeNotifier.RegisterKernelChangeNotify(Self, AllKernelNotifiers);
      Options := Options + [eloChangeNotifierThread];
      if Assigned(RootFolderNamespace) then
        ChangeNotifier.NotifyWatchFolder(Self, RootFolderNamespace.NameForParsing);
      if Assigned(ColumnModeView) then
        ColumnModeView.ChangeNotifierEnabled := Value
    end else
    begin
      ChangeNotifier.UnRegisterKernelChangeNotify(Self);
      Options := Options - [eloChangeNotifierThread];
      if Assigned(ColumnModeView) then
        ColumnModeView.ChangeNotifierEnabled := Value
    end
  end
end;

procedure TUltraExplorerEasyListview.SetShowInactiveEx(const Value: Boolean);
begin
  if Assigned(ColumnModeView) then
  begin
    ColumnModeView.ShowInactive := Value
  end else
    ShowInactive := Value
end;

procedure TUltraExplorerEasyListview.WMDestroy(var Msg: TMessage);
begin
  ChangeNotifierEnabled := False;
  if Assigned(ColumnModeView) then
    ColumnModeView.ChangeNotifierEnabled := False;
  inherited;
end;

procedure TUltraExplorerEasyListview.WMGetIShellBrowser(var Msg: TMessage);
begin
  Msg.Result := HResult(Self as IShellBrowser)
end;

procedure TUltraExplorerEasyListview.WMKillFocus(var Msg: TWMKillFocus);
begin
  DraggingFromBackground := False;
  inherited;
end;

procedure TUltraExplorerEasyListview.WMMouseActivate(var Msg: TWMMouseActivate);
var
  Item: TEasyItem;
  HitInfo: TEasyItemHitTestInfoSet;
  Pt: TPoint;
begin
  // don't call inherited so the message will not be relayed back to the ListMode and it
  // returns it wants the focus.  See the ancestor method for the reason for CheckFocus

  HitInfo := [];
  Pt := Scrollbars.MapWindowToView(ScreenToClient(Mouse.CursorPos));
  Item := Groups.ItemByPoint(Pt);
  if Assigned(Item) then
  begin
    Item.HitTestAt(Pt, HitInfo);
    DraggingFromBackground := [ehtOnIcon, ehtOnText] * HitInfo <> []
  end;

  if DraggingFromBackground then
    Msg.Result := MA_NOACTIVATE
  else begin
    if not EditManager.Editing then
    begin
      CheckFocus;
      Msg.Result := MA_ACTIVATE;
    end else
      Msg.Result := MA_NOACTIVATE;
  end
end;

procedure TUltraExplorerEasyListview.WMNCCalcSize(var Msg: TWMNCCalcSize);
begin
  inherited;
end;

{ TUltraColumnModeView }
constructor TUltraColumnModeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FComponentStyle := [csTransient];
end;

destructor TUltraColumnModeView.Destroy;
begin
  inherited Destroy;
end;

procedure TUltraColumnModeView.SetEnableGesture(const Value: Boolean);
var
  i: Integer;
begin
  if FEnableGesture <> Value then
  begin
    FEnableGesture := Value;
    if Value then
    begin
      for i := 0 to ViewCount - 1 do
      begin
        Views[i].Gesture.Enabled := True;
        Views[i].Selection.MouseButtonSelRect := Views[i].Selection.MouseButtonSelRect - [cmbRight]
      end
    end else
    begin
      for i := 0 to ViewCount - 1 do
      begin
        Views[i].Gesture.Enabled := False;
        Views[i].Selection.MouseButtonSelRect := Views[i].Selection.MouseButtonSelRect + [cmbRight]
      end
    end
  end
end;

procedure TUltraColumnModeView.SetChangeNotifierEnabled(const Value: Boolean);
var
  i: Integer;
begin
  if FChangeNotifierEnabled <> Value then
  begin
    FChangeNotifierEnabled := Value;
    if Value then
    begin
      Options := Options + [eloChangeNotifierThread];
      for i := 0 to ViewCount - 1 do
      begin
        Views[i].Options := Views[i].Options + [eloChangeNotifierThread];
        ChangeNotifier.RegisterKernelChangeNotify(Views[i], AllKernelNotifiers);
      end
    end else
    begin
      Options := Options - [eloChangeNotifierThread];
      for i := 0 to ViewCount - 1 do
      begin
        Views[i].Options := Views[i].Options - [eloChangeNotifierThread];
        ChangeNotifier.UnRegisterKernelChangeNotify(Views[i]);
      end
    end
  end
end;

{ TUltraFilterItem }
constructor TUltraFilterItem.Create(ACollection: TEasyCollection);
begin
  inherited Create(ACollection);
end;


function TUltraFilterItem.GetFilterEnabled: Boolean;
begin
  Result := Checked
end;

function TUltraFilterItem.GetMask: WideString;
begin
  Result := Caption
end;

procedure TUltraFilterItem.LoadFromStream(S: TStream; var Version: Integer);
var
  Temp: TUltraFilterDir;
begin
  inherited LoadFromStream(S, Version);
  // We don't use a FilterDirection on each filter anymore
  if Version = 3 then
    S.Read(Temp, SizeOf(Temp));
  S.Read(FFilterKind, SizeOf(FFilterKind));
end;

procedure TUltraFilterItem.SaveToStream(S: TStream; Version: Integer);
begin
  inherited SaveToStream(S, Version);
  S.Write(FFilterKind, SizeOf(FFilterKind));
end;

procedure TUltraFilterItem.SetFilterEnabled(const Value: Boolean);
begin
  Checked := Value
end;

constructor TUltraInPlacePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TUltraInPlacePanel.Destroy;
begin
  inherited Destroy;
end;

procedure TUltraInPlacePanel.SetInPlaceWindow(const Value: IUltraInPlaceWindow);
begin
  if Value <> FInPlaceWindow then
  begin
    if HandleAllocated then
      SendMessage(Handle, WM_SETREDRAW, 0, 0);
    try
      if Assigned(FInPlaceWindow) then
      begin
        FInPlaceWindow.RegisterNotifier(nil);
        FInPlaceWindow.ShowWindow(False);
        FInPlaceWindow.DestroyWindow;
        FInPlaceWindow := nil;
      end;
      FInPlaceWindow := Value;
      if Assigned(FInPlaceWindow) then
      begin
        if HandleAllocated then
          FInPlaceWindow.ShowWindow(True)
      end
    finally
      if HandleAllocated then
        SendMessage(Handle, WM_SETREDRAW, 1, 0);
    end
  end
end;

{ TUltraEmptyPanel }
procedure TUltraInPlacePanel.WMEraseBkgnd(var Msg: TWMEraseBkgnd);
begin
  Msg.Result := 1;
end;

{ TUltraViewReportItem }    
procedure TUltraViewReportItem.PaintBefore(Item: TEasyItem; Column: TEasyColumn; const Caption: WideString; ACanvas: TCanvas; RectArray: TEasyRectArrayObject; var Handled: Boolean);
var
  R: TRect;
  Scale: Double;
begin
  if (Attribs.Listview.FolderSizeDisplay = bsCustom) and
     (TUltraExplorerItem(Item).FolderSize > -1) and
     (TUltraExplorerEasyListview( Item.OwnerListview).FolderSizeMax > 0) and
     (Column.Index = ID_SIZE_COLUMN) and  
     TUltraExplorerItem(Item).Namespace.Folder and
     not TUltraExplorerItem(Item).Namespace.Browsable and
     TUltraExplorerItem(Item).Namespace.Parent.FileSystem then
  begin
    Handled := True;
    R := RectArray.BoundsRect;
    InflateRect(R, -1, -1);

    if uisThreadLoaded in TUltraExplorerItem(Item).UltraStates then
    begin
      if OwnerListview.Themes.Loaded then
      begin
        LoadTextFont(Item, Column.Index, ACanvas, False);
        InflateRect(R, 0, -(RectHeight(R) - TextExtentW('Tg', ACanvas).cy));

        DrawThemeBackground(OwnerListview.Themes.ProgressTheme, ACanvas.Handle, PP_BAR, 0, R, nil);
        GetThemeBackgroundContentRect(OwnerListview.Themes.ProgressTheme, ACanvas.Handle, PP_BAR, 0, R, @R);
    //    DrawThemeBackground(OwnerListview.Themes.EditThemeTheme, ACanvas.Handle, EP_EDITTEXT, ETS_NORMAL, R, nil);
    //    GetThemeBackgroundContentRect(OwnerListview.Themes.EditThemeTheme, ACanvas.Handle, EP_EDITTEXT, ETS_NORMAL, R, @R);
        Scale := RectWidth(R) / TUltraExplorerEasyListview( Item.OwnerListview).FolderSizeMax;
        R.Right := R.Left + Round(TUltraExplorerItem(Item).FolderSize * Scale);
        FillGradient(R.Left, R.Top, R.Right, R.Bottom, clLime, clGreen, 0, R.Right, ACanvas);
      end else
      begin
        ACanvas.FillRect(R);
        DrawEdge(ACanvas.Handle, R, EDGE_SUNKEN, BF_RECT);
        InflateRect(R, -2, -2);

        Scale := RectWidth(R) / TUltraExplorerEasyListview( Item.OwnerListview).FolderSizeMax;
        R.Right := R.Left + Round(TUltraExplorerItem(Item).FolderSize * Scale);

        ACanvas.Brush.Color := clLime;
        ACanvas.FillRect(R);
      end
    end else
    begin
      RectArray.TextRect := R;
      PaintText(Item, Column, STR_LOADING, RectArray, ACanvas, 1);
    end
  end else
    inherited PaintBefore(Item, Column, Caption, ACanvas, RectArray, Handled);
end;

{ TUltraFolderSizeThreadRequest }
function TUltraFolderSizeThreadRequest.HandleRequest: Boolean;
var
  NS: TNamespace;
begin
  Result := True;
  NS := TNamespace.Create(PIDL, nil);
  try
    NS.FreePIDLOnDestroy := False;
    FolderSize := NS.FolderSize(True, True)
  finally
    NS.Free
  end
end;

{ TUltraExplorerItem }
constructor TUltraExplorerItem.Create(ACollection: TEasyCollection);
begin
  inherited Create(ACollection);
  FFolderSize := -1
end;

{ TUltraFileSearch }

destructor TUltraFileSearch.Destroy;
begin
  inherited Destroy;
end;

function TUltraFileSearch.Run: Boolean;
var
  WS: WideString;
  i: Integer;
begin
  if samFileMask in SearchData.Masks then
  begin
    SearchCriteriaFilename.Text := SearchData.FileNameMask;
    WS := WideLF;
    WS := WS + WideCR;
    SearchCriteriaFilename.Text := Tnt_WideStringReplace(SearchCriteriaFilename.Text, ';', WS, [rfReplaceAll, rfIgnoreCase]);
    SearchCriteriaFilename.Text := Tnt_WideStringReplace(SearchCriteriaFilename.Text, ',', WS, [rfReplaceAll, rfIgnoreCase]);
    for i := SearchCriteriaFilename.Count - 1 downto 0 do
    begin
       SearchCriteriaFilename[i] := Trim(SearchCriteriaFilename[i]);
       if SearchCriteriaFilename[i] = '' then
         SearchCriteriaFilename.Delete(i)
    end
  end else
    SearchCriteriaFilename.Text := '';

{  if samContentMask in SearchData.Masks then
  begin
    SearchCriteriaContent.Text := SearchData.FileContentMask;
    WS := WideLF;
    WS := WS + WideCR;
    SearchCriteriaContent.Text := Tnt_WideStringReplace(SearchCriteriaContent.Text, ';', WS, [rfReplaceAll, rfIgnoreCase]);
    SearchCriteriaContent.Text := Tnt_WideStringReplace(SearchCriteriaContent.Text, ',', WS, [rfReplaceAll, rfIgnoreCase]);
    for i := SearchCriteriaContent.Count - 1 downto 0 do
    begin
       SearchCriteriaContent[i] := Trim(SearchCriteriaContent[i]);
       if SearchCriteriaContent[i] = '' then
         SearchCriteriaContent.Delete(i)
    end
  end else
    SearchCriteriaContent.Text := '';   }

  if (SearchCriteriaFilename.Text <> '') {or (SearchCriteriaContent.Text <> '') }then
  begin
    if SearchCriteriaFilename.Count = 0 then
      SearchCriteriaFilename.Add('*.*');
    SearchPaths.Assign(SearchData.SearchFolders);
    SearchAttribs := SearchData.AdvOptions;
    SubFolders := SearchData.RecurseSubFolder;
    CaseSensitive := SearchData.CaseSensitive;
    ThreadPriority := tpNormal;
    Result := inherited Run;
  end else
    Result := False
end;

procedure TUltraFileSearch.Stop;
begin
  inherited Stop;
end;

end.
