unit UnitBasePlugInForm;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  TntForms,
  Dialogs,
  ImgList,
  ShlObj,
  UnitInPlaceWndInf;

type
  TBasePluginForm = class(TTntForm, IUnknown, IUltraInPlaceWindow, IUltraInplaceNotifications)
  private
    { Private declarations }
    FNotifier: IUltraNotifier;
    FRefCount: Integer;
  protected
    procedure DoClose(var Action: TCloseAction); override;
    property RefCount: Integer read FRefCount write FRefCount;
  public
    { Public declarations }

    // IUnknown
    function QueryInterface(const IID: TGUID; out Obj): HResult; override; stdcall;
    function _AddRef: Integer; reintroduce; stdcall;
    function _Release: Integer; reintroduce; stdcall;

    // IUltraInPlaceWindow
    function ActionOperation(Action: TInPlaceWindowAction): Boolean; virtual;
    function DestroyWindow: Boolean; virtual;
    function GetArrange(var Arrange: TInPlaceWindowArrange): Boolean; virtual;
    function GetGrouped(var Grouped: Boolean): Boolean; virtual;
    function GetHandle(var WindowHandle: hWnd): Boolean; virtual;
    function GetObject(var APlugin: TForm): Boolean; virtual;
    function GetView(var WindowView: TInPlaceWindowView): Boolean; virtual;
    function HandleMessage(var Msg: TMsg; var Handled: Boolean): Boolean; virtual;
    function HintMsg(var AHint: WideString): Boolean; virtual;
    function IsActionEnabled(var InPlaceWindowAction: TInPlaceWindowActions): Boolean; virtual;
    function IsQuickFiltered: Boolean; virtual;
    function LoadConfig: Boolean; virtual;
    function RegisterNotifier(ANotifier: IUltraNotifier): Boolean; virtual;
    function SaveConfig: Boolean; virtual;
    function SetArrange(Arrange: TInPlaceWindowArrange): Boolean; virtual;
    function SetGrouped(Grouped: Boolean): Boolean; virtual;
    function SetParentWnd(AParent: TWinControl): Boolean; virtual;
    function SetQuickFilter(Enable: Boolean; FilterMask: WideString): Boolean; virtual;
    function SetTheme(ThemeName: PChar): Boolean;  virtual;
    function SetView(View: TInPlaceWindowView): Boolean; virtual;
    function SetWindowBounds(AWidth, AHeight: Integer): Boolean; virtual;
    function ShowWindow(Show: Boolean): Boolean; virtual;
    function StatusBarInfo(var ItemCount: Cardinal; var SelectedItemCount: Cardinal; var SelectedItemSize: Int64): Boolean; virtual;
    function TabCaption(var TabCaption: WideString): Boolean; virtual;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; virtual;

    // IUltraInplaceNotifications
    procedure RootFolderChanged(Root: PItemIDList); virtual;
    procedure TabChanging(TabState: TUltraInplaceNotifyTabState); virtual;

    property Notifier: IUltraNotifier read FNotifier write FNotifier;
  end;

implementation

function TBasePluginForm.DestroyWindow: Boolean;
begin
  DestroyHandle;
  Result := True
end;

function TBasePluginForm.GetArrange(var Arrange: TInPlaceWindowArrange): Boolean;
begin
  Result := False
end;

function TBasePluginForm.GetGrouped(var Grouped: Boolean): Boolean;
begin
  Result := False
end;

function TBasePluginForm.GetHandle(var WindowHandle: hWnd): Boolean;
begin
  WindowHandle := 0;
  if HandleAllocated then
  begin
    WindowHandle := Handle;
    Result := True
  end else
    Result := False
end;

function TBasePluginForm.GetObject(var APlugin: TForm): Boolean;
begin
  APlugin := Self;
  Result := True
end;

function TBasePluginForm.GetView(var WindowView: TInPlaceWindowView): Boolean;
begin
  Result := False
end;

function TBasePluginForm.HandleMessage(var Msg: TMsg;
  var Handled: Boolean): Boolean;
begin
  Handled := False;
  Result := True;
end;

function TBasePluginForm.HintMsg(var AHint: WideString): Boolean;
begin
  Result := True;
  AHint := Hint
end;

function TBasePluginForm.LoadConfig: Boolean;
begin
  Result := False
end;

function TBasePluginForm.RegisterNotifier(ANotifier: IUltraNotifier): Boolean;
begin
  // Sends an valid IUltraNotifier when the plugin is created and a nil when
  // the Notifier Sink is being destroyed or the plugin is being destroyed
  Notifier := ANotifier;
  Result := True
end;

function TBasePluginForm.SaveConfig: Boolean;
begin
  Result := False;
end;

function TBasePluginForm.SetArrange(Arrange: TInPlaceWindowArrange): Boolean;
begin
  Result := False
end;

function TBasePluginForm.SetGrouped(Grouped: Boolean): Boolean;
begin
  Result := False
end;

function TBasePluginForm.SetParentWnd(AParent: TWinControl): Boolean;
begin
  Parent := AParent;
  BorderStyle := bsNone;
  Align := alClient;
  Result := Parent = AParent
end;

function TBasePluginForm.SetQuickFilter(Enable: Boolean; FilterMask: WideString): Boolean;
begin
  Result := False
end;

function TBasePluginForm.SetTheme(ThemeName: PChar): Boolean;
begin
  // Use if Form is in DLL where a new TBXSwitcher is needed
  Result := True
end;

function TBasePluginForm.SetView(View: TInPlaceWindowView): Boolean;
begin
  Result := False
end;

function TBasePluginForm.SetWindowBounds(AWidth, AHeight: Integer): Boolean;
begin
  Width := AWidth;
  Height := AHeight;
  Result := (AWidth = Width) and (AHeight = Height)
end;

function TBasePluginForm.ShowWindow(Show: Boolean): Boolean;
begin
  Visible := Show;
  Result := Visible = Show
end;

function TBasePluginForm.TabCaption(var TabCaption: WideString): Boolean;
begin
  Result := True;
  TabCaption := Caption
end;

function TBasePluginForm._AddRef: Integer;
begin
  InterlockedIncrement(FRefCount);
  Result := RefCount;
end;

function TBasePluginForm._Release: Integer;
begin
  InterlockedDecrement(FRefCount);
  Result := RefCount;
  if FRefCount = 0 then
    Free
end;

procedure TBasePluginForm.DoClose(var Action: TCloseAction);
begin
  Action := caFree
end;

procedure TBasePluginForm.TabChanging(TabState: TUltraInplaceNotifyTabState);
begin
 // case TabState of
 //   ntsCreating:    ShowMessage('Tab Creating');
 //   ntsActivating:  ShowMessage('Tab Activating');
 //   ntsDeActivating:ShowMessage('Tab Deactivating');
 //   ntsDestroying:  ShowMessage('Tab Destroying, Save your work');
 // end;
end;

function TBasePluginForm.TabImage(var AnImageList: TCustomImageList;
  var AnImageIndex: Integer): boolean;
begin
  Result := False;
end;

procedure TBasePluginForm.RootFolderChanged(Root: PItemIDList);
begin

end;

function TBasePluginForm.QueryInterface(const IID: TGUID;
  out Obj): HResult;
begin
  if GetInterface(IID, Obj) then Result := S_OK
    else Result := E_NOINTERFACE
end;

function TBasePluginForm.StatusBarInfo(var ItemCount: Cardinal; var SelectedItemCount: Cardinal; var SelectedItemSize: Int64): Boolean;
begin
  Result := False;
end;

function TBasePluginForm.IsQuickFiltered: Boolean;
begin
  Result := False;
end;

function TBasePluginForm.IsActionEnabled(
  var InPlaceWindowAction: TInPlaceWindowActions): Boolean;
begin
  InPlaceWindowAction := [];
  Result := False;
end;

function TBasePluginForm.ActionOperation(Action: TInPlaceWindowAction): Boolean;
begin
  Result := False
end;

end.
