unit UnitBreadcrumbBar;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  MPShellUtilities,
  ShellAPI,
  ShlObj,
  SpTBXControls,
  SpTBXItem,
  SpTBXSkins,
  TB2Item,
  SpTBXDkPanels,
  MPCommonObjects,
  UltraUserStrings,
  TB2Dock,
  TB2Toolbar,
  Graphics,
  MPCommonUtilities,
  Controls,
  ImgList,
  Forms;

const
  ID_HOVER_TIMER = 159;
  HOVER_TIMER_DELAY = 500; // ms

type

  PListModeLayoutData = ^TListModeLayoutData;
  TListModeLayoutData = record
    Splitter: TSpTBXSplitter;
    NS: TNamespace;
    iPosition: Integer;
  end;

  TGlobalEnumFolder = class
  public
    function Enum(MessageWnd: HWnd; APIDL: PItemIDList; AParent: TNamespace; Data: Pointer; var Terminate: Boolean): Boolean;
  end;

  TTBXSubmenuEnumItem = class(TSpTBXSubmenuItem)
  private
    FNS: TNamespace;
  public
    destructor Destroy; override;
    property NS: TNamespace read FNS write FNS;
  end;

  TTBXEnumItem = class(TSpTBXItem)
  private
    FNS: TNamespace;
  public
    destructor Destroy; override;
    property NS: TNamespace read FNS write FNS;
  end;

  PEnumFolderInfo = ^TEnumFolderInfo;
  TEnumFolderInfo = record
    ParentItem: TTBCustomItem;
    ShowFullPath: Boolean;
    OnItemClick: TNotifyEvent;
    OnItemPopup: TTBPopupEvent;
    ShowImages: Boolean;
    ShowText: Boolean;
    DrivesOnly: Boolean;
    ExpandableSubFolders: Boolean;
    ShowBrowsable: Boolean;
  end;

  TBreadCrumbItemClick = procedure(Toobar: TSpTBXToolbar; Item: TTBCustomItem) of object;

  TBreadcrumbBar = class(TSpTBXToolbar)
  private
    FAutoOpenOnHover: Boolean;
    FEnumFlags: TFileObjects;
    FHoverDelayTime: Integer;
    FHoverTimer: Integer;
    FOnItemClick: TBreadCrumbItemClick;
    FShowBrowsable: Boolean;
  protected
    procedure ChevronPopup(Sender: TTBCustomItem; FromLink: Boolean);
    procedure DoItemClick(Item: TTBCustomItem; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure ItemClick(Sender: TObject);
    procedure ItemPopup(Sender: TTBCustomItem; FromLink: Boolean);
    procedure KillHoverTimer;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure SetHoverTimer;
    procedure WMDestroy(var Msg: TMessage); message WM_DESTROY;
    procedure WMTimer(var Msg: TWMTimer); message WM_TIMER;
    procedure WMWindowPosChanged(var Msg: TWMWindowPosChanged); message WM_WINDOWPOSCHANGED;
    property HoverTimer: Integer read FHoverTimer write FHoverTimer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BuildCrumbTrail(NS: TNamespace);
    procedure DropMenu(OldMousePos: TPoint);
    property EnumFlags: TFileObjects read FEnumFlags write FEnumFlags;
    property ShowBrowsable: Boolean read FShowBrowsable write FShowBrowsable;
  published
    property AutoOpenOnHover: Boolean read FAutoOpenOnHover write FAutoOpenOnHover;
    property HoverDelayTime: Integer read FHoverDelayTime write FHoverDelayTime;
    property OnItemClick: TBreadCrumbItemClick read FOnItemClick write FOnItemClick;
  end;

var
  EnumFolder: TGlobalEnumFolder;

implementation

uses
  TypInfo;

procedure SetTBItemCaption(Item: TTBCustomItem; Caption: WideString);
// Set the unicode caption to the Item if it has a valid
// WideString Caption property.
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(Item, 'Caption', [tkWString]);
  if PropInfo = nil then
    Item.Caption := Caption
  else
    SetWideStrProp(Item, PropInfo, Caption);
end;

{ TBreadcrumbBar }
constructor TBreadcrumbBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ChevronMoveItems := False;
  Stretch := True;
  HoverDelayTime := HOVER_TIMER_DELAY;
  EnumFlags := [foFolders];
end;

destructor TBreadcrumbBar.Destroy;
begin
  inherited Destroy;
end;

procedure TBreadcrumbBar.BuildCrumbTrail(NS: TNamespace);
var
  i, LastCrumb: Integer;
  Item: TTBXSubmenuEnumItem;
  PIDLList: TCommonPIDLList;
//  Chevron: TSpTBXSubmenuItem;
begin
  PIDLList := TCommonPIDLList.Create;
  BeginUpdate;
  try
    Items.Clear;

 {   Chevron := TSpTBXSubmenuItem.Create(Self);
    Chevron.Visible := True;
    Chevron.Caption := '<';
    SetTBItemCaption(Chevron, '<');
    Chevron.OnPopup := ChevronPopup;
    Items.Add(Chevron);   }

    // Split up the current Root PIDL to make the bar
    PIDLMgr.ParsePIDL(NS.AbsolutePIDL, PIDLList, True);

    // Create the Desktop Crumb
    Item := TTBXSubmenuEnumItem.Create(Items);
    Item.Options := [tboDropdownArrow];
    Item.NS := TNamespace.Create(nil, nil);
    Item.Caption := Item.NS.NameInFolder;
    SetTBItemCaption(Item, Item.NS.NameInFolder);
    Item.DropdownCombo := True;
    Item.OnClick := ItemClick;
    Item.OnPopup := ItemPopup;
    Items.Add(Item);

    if NS.Folder then
      LastCrumb := PIDLList.Count - 1
    else
      LastCrumb := PIDLList.Count - 2;

    for i := 0 to LastCrumb do
    begin
      Item := TTBXSubmenuEnumItem.Create(Items);
      Item.NS := TNamespace.Create(PIDLMgr.CopyPIDL(PIDLList[i]), nil);
      Item.Caption := Item.NS.NameInFolder;
      SetTBItemCaption(Item, Item.NS.NameInFolder);
      Item.Options := [tboDropdownArrow];
      Item.DropdownCombo := True;
      Item.OnPopup := ItemPopup;
      Item.OnClick := ItemClick;
      Items.Add(Item);
    end
  finally
    EndUpdate;
    PIDLList.Free;
  end
end;

procedure TBreadcrumbBar.ChevronPopup(Sender: TTBCustomItem; FromLink: Boolean);
var
  NewItem: TTBXEnumItem;
  i: Integer;
begin
  Sender.Clear;
  for i := 0 to Items.Count - 1 do
  begin
    if not Items[i].Visible then
    begin
      NewItem := TTBXEnumItem.Create(Sender);
      NewItem.NS := (Items[i] as TTBXEnumItem).NS.Clone(True);
      NewItem.Caption := NewItem.NS.NameNormal;
      SetTBItemCaption(NewItem, NewItem.NS.NameNormal);
      NewItem.ImageIndex := NewItem.NS.GetIconIndex(False, icSmall);
      NewItem.Images := SmallSysImages;
      NewItem.OnClick := ItemClick;
    end
  end;
end;

procedure TBreadcrumbBar.DoItemClick(Item: TTBCustomItem; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(OnItemClick) then
    OnItemClick(Self, Item)
end;

procedure TBreadcrumbBar.DropMenu(OldMousePos: TPoint);
var
  AMsg: TMsg;
  Pos: TPoint;
  Viewer: TTBItemViewer;
begin
  KillHoverTimer;
  if not (vsModal in View.State) then
  begin
    Viewer := View.ViewerFromPoint(ScreenToclient(Mouse.CursorPos));
    if Assigned(Viewer) then
    begin
      Pos := Viewer.BoundsRect.BottomRight;
      Windows.ClientToScreen(Handle, Pos);     
      Dec(Pos.X, 2);
      Dec(Pos.Y, 2);
      SetCursorPos(Pos.X, Pos.Y);
      View.Selected := Viewer;
      // Force system update to GetMessagePos so View will get the
      // new mouse position
      GetMessage(AMsg, 0, 0, 0);
      TranslateMessage(AMsg);
      DispatchMessage(AMsg);
      SetCursorPos(OldMousePos.X, OldMousePos.Y);
      View.EnterToolbarLoop([tbetMouseDown]);
    end
  end;
end;

procedure TBreadcrumbBar.ItemClick(Sender: TObject);
begin
  DoItemClick(Sender as TTBCustomItem, mbLeft, [], 0, 0)
end;

procedure TBreadcrumbBar.ItemPopup(Sender: TTBCustomItem; FromLink: Boolean);
var
  Info: TEnumFolderInfo;
  OldMode: UINT;
  Item: TTBXSubmenuEnumItem;
  NewItem: TTBXEnumItem;
begin
  Sender.Clear;
  FillChar(Info, SizeOf(Info), #0);
  Item := Sender as TTBXSubmenuEnumItem;
  Info.ParentItem := Item;
  Info.OnItemClick := ItemClick;
  Info.OnItemPopup := ItemPopup;
  Info.ShowImages := True;
  Info.ShowText := True;
  Info.ExpandableSubFolders := True;
  Info.ShowBrowsable := ShowBrowsable;
  
  OldMode := SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    Sender.SubMenuImages := SmallSysImages;
    Item.NS.EnumerateFolder(0, True, False, foHidden in EnumFlags, EnumFolder.Enum, @Info);
    Item.Add(TSpTBXSeparatorItem.Create(Item));
    NewItem := TTBXEnumItem.Create(Item);
    NewItem.NS := Item.NS.Clone(True);
    NewItem.Caption := STR_PROPERTIES;
    NewItem.Tag := 1; // Signal this is not a normal item
    SetTBItemCaption(NewItem, STR_PROPERTIES);
    NewItem.OnClick := ItemClick;
    Item.Add(NewItem);
  finally
    SetErrorMode(OldMode)
  end
end;

procedure TBreadcrumbBar.KillHoverTimer;
begin
  if HoverTimer <> 0 then
  begin
    KillTimer(Handle, ID_HOVER_TIMER);
    HoverTimer := 0
  end
end;

procedure TBreadcrumbBar.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  SetHoverTimer;
end;

procedure TBreadcrumbBar.SetHoverTimer;
begin
  KillHoverTimer;
  if AutoOpenOnHover then
    HoverTimer := SetTimer(Handle, ID_HOVER_TIMER, abs(HoverDelayTime), nil)
end;

procedure TBreadcrumbBar.WMDestroy(var Msg: TMessage);
begin
  KillHoverTimer;
  inherited
end;

procedure TBreadcrumbBar.WMTimer(var Msg: TWMTimer);
begin
  case Msg.TimerID of
    ID_HOVER_TIMER: DropMenu(Mouse.CursorPos);
  end;
  inherited
end;

procedure TBreadcrumbBar.WMWindowPosChanged(var Msg: TWMWindowPosChanged);
var
  Viewer: TTBItemViewer;
  i, W: Integer;
begin
  inherited;
  if Msg.WindowPos.flags and SWP_NOSIZE	= 0 then
  begin
    W := 0;
    for i := Items.Count - 1 downto 0 do
    begin
      Viewer := View.Find(Items[i]);
      if Assigned(Viewer) then
      begin
        W := W + RectWidth(Viewer.BoundsRect);
        if W > ClientWidth then
   //       beep;
      end
    end
  end
end;

{ TTBXSubmenuCrumbItem }
destructor TTBXSubmenuEnumItem.Destroy;
begin
  NS.Free;
  inherited Destroy;
end;

{ TGlobalEnumFolder}
function TGlobalEnumFolder.Enum(MessageWnd: HWnd; APIDL: PItemIDList; AParent: TNamespace; Data: Pointer; var Terminate: Boolean): Boolean;
var
  Item: TTBCustomItem;
  NS: TNamespace;
  Info: PEnumFolderInfo;
begin
  Result := True;
  Info := PEnumFolderInfo( Data);
  NS := TNamespace.Create(APIDL, AParent);
  if not Info.ShowBrowsable and NS.Browsable then
  begin
    NS.Free
  end else
  begin
    if Info^.DrivesOnly then
    begin
      Result := WideIsDrive(NS.NameParseAddress);
      if not Result then
        FreeAndNil(NS)
    end else
      Result := True;

    if Result then
    begin
      if Info^.ExpandableSubFolders and NS.HasSubFolder then
      begin
        Item := TTBXSubmenuEnumItem.Create(Info^.ParentItem);
        TTBXSubmenuEnumItem(Item).NS := NS;
        TTBXSubmenuEnumItem(Item).DropdownCombo := True;
        TTBXSubmenuEnumItem(Item).OnPopup := Info^.OnItemPopup;
      end else
      begin
        Item := TTBXEnumItem.Create(Info^.ParentItem);
        (Item as TTBXEnumItem).NS := NS;
      end;
      if Info.ShowText then
      begin
        if Info.ShowFullPath then
        begin
          Item.Caption := NS.NameForParsing;
          SetTBItemCaption(Item, NS.NameForParsing);
        end else
        begin
          Item.Caption := NS.NameNormal;
          SetTBItemCaption(Item, NS.NameInFolder);
        end;
      end;
      if Info^.ShowImages then
        Item.ImageIndex := NS.GetIconIndex(False, icSmall);
      Item.OnClick := Info.OnItemClick;
      Info.ParentItem.Add(Item);
    end
  end
end;

{ TTBXEnumItem }
destructor TTBXEnumItem.Destroy;
begin
  NS.Free;
  inherited Destroy;
end;

initialization
  EnumFolder := TGlobalEnumFolder.Create;

finalization
  FreeAndNil(EnumFolder)

end.
