unit UnitEditGroupingCriteriaEdit;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  SpTBXControls,
  StdCtrls,
  SpTBXEditors,
  Math,
  UltraUserStrings,
  VirtualExplorerEasyListview,
  DKLang,
  MPCommonUtilities, SpTBXItem;

type
  TNumericType = (
    ntSingle,
    ntInt64
  );

  TTntFormGroupingCriteriaEdit = class(TTntForm)
    SpTBXPanel1: TSpTBXPanel;
    SpTBXButtonOk: TSpTBXButton;
    SpTBXButtonCancel: TSpTBXButton;
    SpTBXEditCaption: TSpTBXEdit;
    SpTBXEditValue: TSpTBXEdit;
    SpTBXLabel1: TSpTBXLabel;
    SpTBXLabel2: TSpTBXLabel;
    SpTBXButtonMaxValue: TSpTBXButton;
    DKLanguageController1: TDKLanguageController;
    procedure SpTBXEditValueKeyPress(Sender: TObject; var Key: Char);
    procedure SpTBXButtonMaxValueClick(Sender: TObject);
    procedure SpTBXButtonOkClick(Sender: TObject);
    procedure TntFormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TntFormShow(Sender: TObject);
  private
    { Private declarations }
    FNumericType: TNumericType;
    FWndParent: HWnd;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    property WndParent: HWnd read FWndParent write FWndParent;
  public
    { Public declarations }
    constructor CreateEx(AnOwner: TCustomForm);

    property NumericType: TNumericType read FNumericType write FNumericType;
  end;

implementation

{$R *.DFM}

constructor TTntFormGroupingCriteriaEdit.CreateEx(AnOwner: TCustomForm);
begin
  if Assigned(AnOwner) then
     WndParent := AnOwner.Handle
   else
     WndParent := 0;
   inherited Create(nil);
end;

procedure TTntFormGroupingCriteriaEdit.CreateParams(var Params: TCreateParams);
begin
  inherited;
  if WndParent <> 0 then
    Params.WndParent := WndParent;
end;

procedure TTntFormGroupingCriteriaEdit.SpTBXEditValueKeyPress(Sender: TObject; var Key: Char);
begin
  if not ((Key in NUMERIC_KEYS) or ((ntSingle = NumericType) and (Key in FLOAT_KEYS)) or (Key = #8 {Backspace})) then
    Key := #0;
end;

procedure TTntFormGroupingCriteriaEdit.SpTBXButtonMaxValueClick(Sender: TObject);
begin
  case NumericType of
    ntSingle: SpTBXEditValue.Text := FloatToStr(MAXSINGLE);
    ntInt64: SpTBXEditValue.Text := FloatToStr(MAXINT64);
  end;
end;

procedure TTntFormGroupingCriteriaEdit.SpTBXButtonOkClick(Sender: TObject);
begin
  case NumericType of
    ntSingle:
      begin
        if StrToFloat(SpTBXEditValue.Text) > MAXSINGLE then
          SpTBXEditValue.Text := FloatToStr(MAXSINGLE);
      end;
    ntInt64:
      begin
        if StrToInt64(SpTBXEditValue.Text) > MAXINT64 then
        SpTBXEditValue.Text := FloatToStr(MAXINT64);
      end
  end;
end;

procedure TTntFormGroupingCriteriaEdit.TntFormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close
end;

procedure TTntFormGroupingCriteriaEdit.TntFormShow(Sender: TObject);
begin
  FixFormFont(Font);
  if SpTBXButtonOk.Top + SpTBXButtonOk.Height > ClientHeight then
   ClientHeight := SpTBXButtonOk.Top + SpTBXButtonOk.Height + 8
end;

end.
