unit UnitFTPTree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs, VirtualTrees,
  ftpsend, TntSysUtils, TntClasses;

type
  TEasyFTPConnection = class(TCollectionItem)
  private
    FAttempts: Integer;
    FDescription: WideString;
    FHost: WideString;
    FInitalPath: WideString;
    FPassword: WideString;
    FPort: Integer;
    FProfile: WideString;
    FRetryDelay: Integer;
    FUserName: WideString;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Attempts: Integer read FAttempts write FAttempts;
    property Description: WideString read FDescription write FDescription;
    property Host: WideString read FHost write FHost;
    property InitalPath: WideString read FInitalPath write FInitalPath;
    property Password: WideString read FPassword write FPassword;
    property Port: Integer read FPort write FPort;
    property Profile: WideString read FProfile write FProfile;
    property RetryDelay: Integer read FRetryDelay write FRetryDelay;
    property UserName: WideString read FUserName write FUserName;
  end;

  TEasyFTPConnections = class(TCollection)
  private
    function GetItem(Index: Integer): TEasyFTPConnection;
    procedure SetItem(Index: Integer; Value: TEasyFTPConnection);
  public
    function Add: TEasyFTPConnection; reintroduce;
    property Items[Index: Integer]: TEasyFTPConnection read GetItem write SetItem; default;
  end;

  TEasyFTPConnectionsComponent = class(TComponent)
  private
    FItems: TEasyFTPConnections;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure LoadFromFile(FileName: WideString);
    procedure SaveToFile(FileName: WideString);
  published
    property Items: TEasyFTPConnections read FItems write FItems;
  end;
  
  TEasyFTPTreeData = class
  private
    FCaption: WideString;
  public
    property Caption: WideString read FCaption write FCaption;
  end;
  
  TEasyFTPTree = class(TCustomVirtualStringTree)
  private
    FFTP: TFTPSend;
  protected
    procedure DoGetText(Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var Text: WideString); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Connect(Host, Port, UserName, Password: WideString): Boolean;
    function Disconnect: Boolean;
    property FTP: TFTPSend read FFTP write FFTP;
  end;

var
  FTPConnections: TEasyFTPConnectionsComponent = nil;

implementation

{ TEasyFTPTree }
constructor TEasyFTPTree.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  NodeDataSize := SizeOf(TEasyFTPTreeData);
  FTP := TFTPSend.Create;
end;

destructor TEasyFTPTree.Destroy;
begin
  FreeAndNil(FFTP);
  inherited Destroy;
end;

function TEasyFTPTree.Connect(Host, Port, UserName, Password: WideString): Boolean;
var
  Data: TEasyFTPTreeData;
begin
  FTP.TargetHost := Host;
  FTP.TargetPort := Port;
  FTP.UserName := UserName;
  FTP.Password := Password;
  Result := FTP.Login;
  if Result then
  begin
    Data := TEasyFTPTreeData.Create;
    Data.Caption := FTP.TargetHost;
    AddChild(nil, Data)
  end;
end;

function TEasyFTPTree.Disconnect: Boolean;
begin
  {Result :=} FTP.Logout
end;

procedure TEasyFTPTree.DoGetText(Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType; var Text: WideString);
var
  Data: TEasyFTPTreeData;
begin
  Data := TEasyFTPTreeData( InternalData(Node));
  Text := Data.Caption;
  inherited DoGetText(Node, Column, TextType, Text);
end;

{ TEasyFTPConnection }
procedure TEasyFTPConnection.Assign(Source: TPersistent);
var
  Connection: TEasyFTPConnection;
begin
  if Source is TEasyFTPConnection then
  begin
    Connection := TEasyFTPConnection( Source);
    Attempts := Connection.Attempts;
    Description := Connection.Description;
    Host := Connection.Host;
    InitalPath := Connection.InitalPath;
    Password := Connection.Password;
    Port := Connection.Port;
    Profile  := Connection.Profile;
    RetryDelay := Connection.RetryDelay;
    UserName := Connection.UserName;
  end
end;

{ TEasyFTPConnections }
function TEasyFTPConnections.Add: TEasyFTPConnection;
begin
  Result := TEasyFTPConnection(inherited Add)
end;

function TEasyFTPConnections.GetItem(Index: Integer): TEasyFTPConnection;
begin
  Result := TEasyFTPConnection( inherited Items[Index])
end;

procedure TEasyFTPConnections.SetItem(Index: Integer; Value: TEasyFTPConnection);
begin
  inherited Items[Index] := Value
end;

{ TEasyFTPConnectionsComponent }
constructor TEasyFTPConnectionsComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TEasyFTPConnections.Create(TEasyFTPConnection);
end;

destructor TEasyFTPConnectionsComponent.Destroy;
begin
  FreeAndNIl(FItems);
  inherited Destroy;
end;

procedure TEasyFTPConnectionsComponent.Assign(Source: TPersistent);
begin
  if Source is TEasyFTPConnectionsComponent then
  begin
    Items.Assign(TEasyFTPConnectionsComponent( Source).Items)
  end
end;

procedure TEasyFTPConnectionsComponent.LoadFromFile(FileName: WideString);
var
  FileStream: TTntFileStream;
  BinaryStream: TMemoryStream;
begin
  BinaryStream := TMemoryStream.Create;
  FileStream := TTntFileStream.Create(FileName, fmOpenRead or fmShareExclusive);
  try
    ObjectTextToBinary(FileStream, BinaryStream);
    BinaryStream.Seek(0, soFromBeginning);
    BinaryStream.ReadComponent(Self);
  finally
    FileStream.Free;
    BinaryStream.Free
  end
end;

procedure TEasyFTPConnectionsComponent.SaveToFile(FileName: WideString);
var
  FileStream: TTntFileStream;
  BinaryStream: TMemoryStream;
begin
  BinaryStream := TMemoryStream.Create;
  FileStream := TTntFileStream.Create(FileName, fmCreate	or fmShareExclusive);
  try
    BinaryStream.WriteComponent(Self);
    BinaryStream.Seek(0, soFromBeginning);
    ObjectBinaryToText(BinaryStream, FileStream);
  finally
    FileStream.Free;
    BinaryStream.Free
  end
end;

initialization
  FTPConnections := TEasyFTPConnectionsComponent.Create(nil);

finalization
  FreeAndNIL(FTPConnections);

end.
