unit UnitFavoriteCrumbBar;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  MPShellUtilities,
  ShellAPI,
  ShlObj,
  SpTBXControls,
  SpTBXItem,
  SpTBXSkins,
  TB2Item,
  MPCommonObjects,
  UltraUserStrings;

type
  TFavoriteItem = class
  private
    FItemList: TList;
    FNS: TNamespace;  // If nil then the item is a Favorite Folder
    function GetItems(Index: Integer): TFavoriteItem;
    property ItemList: TList read FItemList write FItemList;
  protected
    
  public
    constructor Create(PIDL: PItemIDList); virtual;
    destructor Destroy; override;
    function AddChild(PIDL: PItemIDList): TFavoriteItem;
    procedure Clear;
    property Items[Index: Integer]: TFavoriteItem read GetItems; default;
    property NS: TNamespace read FNS write FNS;
  end;


implementation

uses
  TypInfo;

procedure SetTBItemCaption(Item: TTBCustomItem; Caption: WideString);
// Set the unicode caption to the Item if it has a valid
// WideString Caption property.
var
  PropInfo: PPropInfo;
begin
  PropInfo := GetPropInfo(Item, 'Caption', [tkWString]);
  if PropInfo = nil then
    Item.Caption := Caption
  else
    SetWideStrProp(Item, PropInfo, Caption);
end;

{ TFavoriteItem}
constructor TFavoriteItem.Create(PIDL: PItemIDList);
begin
  inherited Create;
  ItemList := TList.Create;
  if Assigned(PIDL) then
    NS := TNamespace.Create(PIDL, nil);
end;

destructor TFavoriteItem.Destroy;
begin
  Clear;
  FreeAndNil(FItemList);
  inherited Destroy;
end;

function TFavoriteItem.AddChild(PIDL: PItemIDList): TFavoriteItem;
begin
  Result := TFavoriteItem.Create(PIDL);
  ItemList.Add(Result)
end;

function TFavoriteItem.GetItems(Index: Integer): TFavoriteItem;
begin
  Result := TFavoriteItem( Items[Index])
end;

procedure TFavoriteItem.Clear;
var
  i: Integer;
begin
  try
    for i := 0 to ItemList.Count - 1 do
      TObject(Items[i]).Free
  finally
    ItemList.Count := 0
  end
end;

end.
