unit UnitInPlaceWndInf;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Classes,
  Controls,
  ActiveX,
  Forms,
  TntForms,
  ImgList,
  ShlObj;

type
  TUltraNotifierUIUpdate = (
    nuiStatusbar,   // The main UI should update the Statusbar
    nuiActionbar    // The main UI should update the Listview Action bar (copy/paste/cut etc)
  );

  //****************************************************************************
  // Attributes to the inplace Window to query what Actions it supports.  Any unsupported
  // actions will be disabled in the UI
  //****************************************************************************

  // ViewEdit Toolbar
  TInPlacetViewEditAttributes = (
    ipeaCopy,             // Supports Copying selected items to the clipboard
    ipeaCut,              // Supports cutting selected items to the clipboard
    ipeaPaste,            // Supports pasting items from the clipboard
    ipeaDelete,           // Supports deleting selected items
    ipeaSelectAll,        // Supports selecting all items
    ipeaInvertSelect,     // Supports inverting the selection
    ipeaRefresh,          // Supports refreshing the plugin
    ipeaProperites        // Supports showing the properites of the selected items
  );

  // ViewAction Toolbar
  TInPlacetViewActionAttributes = (
    ipaaNew,              // Supports creating New items
    ipaaSendTo,           // Supports sending selected items to
    ipaaView,             // Supports different Views (Icon, Small Icon, List, Report, Tile, Thumbnails)
    ipaaGroup,            // Supports Grouping
    ipaaQuickThumbs,      // Supports QuickThumbs (Thumbnail view)
    ipaaStretch,          // Supports stretching the Thumbnails to fit the entire cell
    ipaaCopyPath          // Supports copying the path of selected items to the clipboard
  );

  // Navigation Toolbar
  TInPlaceNavigationAttributes = (
    ipnaBack,             // Supports browsing Back
    ipnaForward,          // Supports browsing Forward
    ipnaUp,               // Supports browsing Up one folder
    ipnaIcon,             // Supports Icon View
    ipnaSmallIcon,        // Supports Small Icon View
    ipnaList,             // Supports List View
    ipnaDetail,           // Supports Details View
    ipnaThumbnail,        // Supports Thumbnail View
    ipnaTile,             // Supports Tile View
    ipnaColumnMode,       // Supports Column View
    ipnaDualPane          // Supports Dual Pane Layout
  );

  // Various types of Tasks
  TInPlaceMiscAttributes = (
    ipmaFileObject,         // Supports setting which file objects to enumerate
    ipmaQuickFilter,        // Supports the QuickFilter bar
    ipmaFilter,             // Supports the Filter Window
    ipmaViewer,             // Supports the Viewer Window
    ipmaBrowsing,           // Supports Browsing to a new folder
    ipmaContextMenu,        // Supports IContextMenu for selected items
    ipmaCalcFolderSize,     // Supports calculating Folders Sizes for Folder objects
    ipmaAutoRefresh         // Supports AutoRefresh (and enabling/disabling it)
  );

  // Statusbar Support
  TInPlaceStatusbarAttributes = (
    ipsaItemCount,         // Supports the Object Count field of the statusbar
    ipsaItemsSelected,     // Supports the Object Selected Count field of the statusbar
    ipsaBytesSelected,     // Supports the Bytes Selected field of the statusbar
    ipsaDriveInfo          // Supports the Drive Info field of the statusbar
  );
  //****************************************************************************


  //****************************************************************************
  // Commands to the inplace Window.
  //****************************************************************************
  // ViewEdit Toolbar
  TInPlacetViewEditCommand = (
    ipecCopy,             // Copy selected items to the clipboard
    ipecCut,              // Cut selected items to the clipboard
    ipecPaste,            // Paste items from the clipboard
    ipecDelete,           // Delete selected items
    ipecSelectAll,        // Select all items
    ipecInvertSelect,     // Invert the selection
    ipecRefresh,          // Refresh the plugin
    ipecProperites        // Show the properites of the selected items
  );

  // ViewAction Toolbar
  TInPlacetViewActionCommand = (
    ipacNew,              // Create New item
    ipacSendTo,           // Send selected items to
    ipacView,             // Set View (Icon, Small Icon, List, Report, Tile, Thumbnails)
    ipacGroup,            // Group
    ipacQuickThumbs,      // Set QuickThumbs (Thumbnail view)
    ipacStretch,          // Stretch the Thumbnails to fit the entire cell
    ipacCopyPath          // Copy the path of selected items to the clipboard
  );

  // Navigation Toolbar
  TInPlaceNavigationCommand = (
    ipncBack,             // Browse Back
    ipncForward,          // Browse Forward
    ipncUp,               // Browse Up one folder
    ipncIcon,             // Set Icon View
    ipncSmallIcon,        // Set Small Icon View
    ipncList,             // Set List View
    ipncDetail,           // Set Details View
    ipncThumbnail,        // Set Thumbnail View
    ipncTile,             // Set Tile View
    ipncColumnMode,       // Set Column View
    ipncDualPane          // Set Dual Pane Layout
  );

  // Various types of Tasks
  TInPlaceMiscCommand = (
    ipmcFileObject,         // Supports setting which file objects to enumerate
    ipmcQuickFilter,        // Supports the QuickFilter bar
    ipmcFilter,             // Supports the Filter Window
    ipmcViewer,             // Supports the Viewer Window
    ipmcBrowsing,           // Supports Browsing to a new folder
    ipmcContextMenu,        // Supports IContextMenu for selected items
    ipmcCalcFolderSize,     // Supports calculating Folders Sizes for Folder objects
    ipmcAutoRefresh         // Supports AutoRefresh (and enabling/disabling it)
  );

  // Statusbar Support
  TInPlaceStatusbarCommand = (
    ipscItemCount,         // Supports the Object Count field of the statusbar
    ipscItemsSelected,     // Supports the Object Selected Count field of the statusbar
    ipscBytesSelected,     // Supports the Bytes Selected field of the statusbar
    ipscDriveInfo          // Supports the Drive Info field of the statusbar
  );
  //****************************************************************************


  IUltraNotifier = interface(IUnknown)
  ['{9713099B-9D1B-4C2F-95FA-F84ECB6BC00B}']
    procedure FocusChanged(NewFocus: TWinControl);
    procedure UpdateUI(UpdateType: TUltraNotifierUIUpdate);
  end;

  IUltraInPlaceWindow = interface(IUnknown)
  ['{BCAAE619-7EC3-4E96-AE58-E2CE86E1DEC0}']
    function DestroyWindow: Boolean;
    function GetHandle(var WindowHandle: hWnd): Boolean;
    function GetObject(var APlugin: TTntFrame): Boolean;
    function HandleMessage(var Msg: TMsg; var Handled: Boolean): Boolean;
    function HintMsg(var AHint: WideString): Boolean;
    function LoadConfig: Boolean;
    function RegisterNotifier(ANotifier: IUltraNotifier): Boolean;
    function SaveConfig: Boolean;
    function SetParentWnd(AParent: TWinControl): Boolean;
    function SetTheme(ThemeName: PChar): Boolean;
    function SetWindowBounds(AWidth, AHeight: Integer): Boolean;
    function ShowWindow(Show: Boolean): Boolean;
    function TabCaption(var ACaption: WideString): Boolean;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean;
  end;

  IUltraInPlaceCommands = interface(IUnknown)
  ['{52CBCF33-3FB3-406A-B523-E262A7348D6B}']
    function CommandViewEdit(Command: TInPlacetViewEditCommand): Boolean;
    function CommandViewAction(Command: TInPlacetViewActionCommand; Info: Pointer): Boolean;
    function CommandNavigation(Command: TInPlaceNavigationCommand): Boolean;
    function CommandMisc(Command: TInPlaceMiscCommand): Boolean;
    function CommandStatusBar(Command: TInPlaceStatusbarCommand; var WideString): Boolean;
  end;

  IUltraInPlaceAttributes = interface(IUnknown)
  ['{D8A5E13A-7DCF-4068-9580-607C018B0598}']
    function AttributesViewEdit(var Attributes: TInPlacetViewEditAttributes): Boolean;
    function AttributesViewAction(var Command: TInPlacetViewActionAttributes): Boolean;
    function AttributesNavigation(var Command: TInPlaceNavigationAttributes): Boolean;
    function AttributesMisc(var Command: TInPlaceMiscAttributes): Boolean;
    function AttributesStatusBar(var Command: TInPlaceStatusbarAttributes): Boolean;
  end;



  TUltraInplaceNotifyTabState = (
    ntsActivating,
    ntsDeActivating,
    ntsDestroying,
    ntsCreating
  );

  IUltraInplaceNotifications = interface(IUnknown)
  ['{5010F3C6-34C7-4CE5-B719-26237E9D6E79}']
    procedure TabChanging(TabState: TUltraInplaceNotifyTabState);
    procedure RootFolderChanged(Root: PItemIDList);
  end;

implementation

end.
