unit UnitOptionsViewer;

// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you maynot use this file except in compliance
// with the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
// Alternatively, you may redistribute this library, use and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation;
// either version 2.1 of the License, or (at your option) any later version.
// You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
// specific language governing rights and limitations under the License.

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  SpTBXControls,
  SpTBXDkPanels,
  StdCtrls,
  SpTBXEditors,
  UnitOptionsBase,
  DKLang, SpTBXItem;

type
  TTntFormOptionsViewer = class(TTntOptionsBaseForm)
    SpTBXPanel1: TSpTBXPanel;
    SpTBXGroupBoxViewerExtensions: TSpTBXGroupBox;
    SpTBXEditViewerMediaExt: TSpTBXEdit;
    SpTBXEditViewerImageExt: TSpTBXEdit;
    SpTBXEditViewerTextExt: TSpTBXEdit;
    SpTBXLabelViewerText: TSpTBXLabel;
    SpTBXLabelViewerMedia: TSpTBXLabel;
    SpTBXLabelViewerImages: TSpTBXLabel;
    SpTBXLabelViewerInternet: TSpTBXLabel;
    SpTBXLabelViewerRTF: TSpTBXLabel;
    SpTBXEditViewerInternetExt: TSpTBXEdit;
    SpTBXEditViewerRTFExt: TSpTBXEdit;
    SpTBXGroupBoxViewerFont: TSpTBXGroupBox;
    SpTBXButtonAnsiFont: TSpTBXButton;
    SpTBXLabelAnsiFont: TSpTBXLabel;
    SpTBXLabelOemFont: TSpTBXLabel;
    SpTBXButtonOemFont: TSpTBXButton;
    SpTBXGroupBoxColors: TSpTBXGroupBox;
    SpTBXPanelTextWindowBkgnd: TSpTBXPanel;
    SpTBXLabel6: TSpTBXLabel;
    SpTBXLabel7: TSpTBXLabel;
    SpTBXLabel8: TSpTBXLabel;
    SpTBXPanelTextWindowHex1: TSpTBXPanel;
    SpTBXPanelTextWindowHex2: TSpTBXPanel;
    SpTBXButtonTextWindowBkgnd: TSpTBXButton;
    SpTBXButtonTextWindowHex1: TSpTBXButton;
    SpTBXButtonTextWindowHex2: TSpTBXButton;
    SpTBXPanelTextWindowHexBkGnd: TSpTBXPanel;
    SpTBXButtonHexBkGnd: TSpTBXButton;
    SpTBXLabel4: TSpTBXLabel;
    DKLanguageController1: TDKLanguageController;
    SpTBXGroupBox1: TSpTBXGroupBox;
    SpTBXCheckBoxAutoPlayOnLoad: TSpTBXCheckBox;
    procedure SpTBXEditViewerTextExtChange(Sender: TObject);
    procedure SpTBXButtonAnsiFontClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpTBXButtonOemFontClick(Sender: TObject);
    procedure SpTBXButtonTextWindowBkgndClick(Sender: TObject);
    procedure SpTBXButtonTextWindowHex1Click(Sender: TObject);
    procedure SpTBXButtonTextWindowHex2Click(Sender: TObject);
    procedure SpTBXButtonHexBkGndClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpTBXIsDirtyClick(Sender: TObject);
  private
    FAnsiFont: TFont;
    FOEMFont: TFont;
    { Private declarations }
  public
    property AnsiFont: TFont read FAnsiFont write FAnsiFont;
    property OEMFont: TFont read FOEMFont write FOEMFont;
  end;

implementation

{$R *.DFM}

procedure TTntFormOptionsViewer.SpTBXEditViewerTextExtChange(
  Sender: TObject);
begin
  Dirty(True)
end;

procedure TTntFormOptionsViewer.SpTBXButtonAnsiFontClick(Sender: TObject);
begin
  FontDialog.Font.Assign(AnsiFont);
  if FontDialog.Execute then
  begin
    AnsiFont.Assign(FontDialog.Font);
    Dirty(True)
  end

end;

procedure TTntFormOptionsViewer.FormCreate(Sender: TObject);
begin
  inherited;
  AnsiFont := TFont.Create;
  OEMFont := TFont.Create;
end;

procedure TTntFormOptionsViewer.FormDestroy(Sender: TObject);
begin
  inherited;
  FreeAndNil(FAnsiFont);
  FreeAndNil(FOEMFont);
end;

procedure TTntFormOptionsViewer.SpTBXButtonOemFontClick(Sender: TObject);
begin
  FontDialog.Font.Assign(OEMFont);
  if FontDialog.Execute then
  begin
    OEMFont.Assign(FontDialog.Font);
    Dirty(True)
  end
end;

procedure TTntFormOptionsViewer.SpTBXButtonTextWindowBkgndClick(
  Sender: TObject);
begin
  ColorDialog.Color := SpTBXPanelTextWindowBkgnd.Color;
  if ColorDialog.Execute then
  begin
    SpTBXPanelTextWindowBkgnd.Color := ColorDialog.Color;
    SpTBXPanelTextWindowBkgnd.InvalidateBackground;
    Dirty(True)
  end
end;

procedure TTntFormOptionsViewer.SpTBXButtonTextWindowHex1Click(
  Sender: TObject);
begin
  ColorDialog.Color := SpTBXPanelTextWindowHex1.Color;
  if ColorDialog.Execute then
  begin
    SpTBXPanelTextWindowHex1.Color := ColorDialog.Color;
    SpTBXPanelTextWindowHex1.InvalidateBackground;
    Dirty(True)
  end
end;

procedure TTntFormOptionsViewer.SpTBXButtonTextWindowHex2Click(
  Sender: TObject);
begin
  ColorDialog.Color := SpTBXPanelTextWindowHex2.Color;
  if ColorDialog.Execute then
  begin
    SpTBXPanelTextWindowHex2.Color := ColorDialog.Color;
    SpTBXPanelTextWindowHex2.InvalidateBackground;
    Dirty(True)
  end
end;

procedure TTntFormOptionsViewer.SpTBXButtonHexBkGndClick(Sender: TObject);
begin
  ColorDialog.Color := SpTBXPanelTextWindowHexBkGnd.Color;
  if ColorDialog.Execute then
  begin
    SpTBXPanelTextWindowHexBkGnd.Color := ColorDialog.Color;
    SpTBXPanelTextWindowHexBkGnd.InvalidateBackground;
    Dirty(True)
  end
end;

procedure TTntFormOptionsViewer.FormShow(Sender: TObject);
begin
  ClientWidth := SpTBXPanel1.Width;
  ClientHeight := SpTBXPanel1.Height;
end;

procedure TTntFormOptionsViewer.SpTBXIsDirtyClick(Sender: TObject);
begin
  inherited;
  Dirty(True);
end;

end.
