unit UnitPlugInBaseFrame;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TntForms,
  ImgList,
  ActiveX,
  ShlObj,
  UnitInPlaceWndInf;

type
  TTntFrameBasePlugIn = class(TTntFrame, IUnknown, IUltraInPlaceWindow, IUltraInplaceNotifications)
  private
    { Private declarations }
    FNotifier: IUltraNotifier;
    FRefCount: Integer;
  protected
    property RefCount: Integer read FRefCount write FRefCount;
  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;
    // IUnknown
    function QueryInterface(const IID: TGUID; out Obj): HResult; override; stdcall;
    function _AddRef: Integer; reintroduce; stdcall;
    function _Release: Integer; reintroduce; stdcall;

    function DestroyWindow: Boolean; virtual;
    function GetHandle(var WindowHandle: hWnd): Boolean; virtual;
    function GetObject(var APlugin: TTntFrame): Boolean; virtual;
    function HandleMessage(var Msg: TMsg; var Handled: Boolean): Boolean; virtual;
    function HintMsg(var AHint: WideString): Boolean; virtual;
    function LoadConfig: Boolean; virtual;
    function RegisterNotifier(ANotifier: IUltraNotifier): Boolean; virtual;
    function SaveConfig: Boolean; virtual;
    function SetParentWnd(AParent: TWinControl): Boolean; virtual;
    function SetTheme(ThemeName: PChar): Boolean;  virtual;
    function SetWindowBounds(AWidth, AHeight: Integer): Boolean; virtual;
    function ShowWindow(Show: Boolean): Boolean; virtual;
    function TabCaption(var ACaption: WideString): Boolean; virtual;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; virtual;

    // IUltraInplaceNotifications
    procedure RootFolderChanged(Root: PItemIDList); virtual;
    procedure TabChanging(TabState: TUltraInplaceNotifyTabState); virtual;

    property Notifier: IUltraNotifier read FNotifier write FNotifier;
  end;

var
  TntFrameBasePlugIn: TTntFrameBasePlugIn;

implementation

{$R *.DFM}

constructor TTntFrameBasePlugIn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TTntFrameBasePlugIn.DestroyWindow: Boolean;
begin
  DestroyHandle;
  Result := True
end;

function TTntFrameBasePlugIn.GetHandle(var WindowHandle: hWnd): Boolean;
begin
  WindowHandle := 0;
  if HandleAllocated then
  begin
    WindowHandle := Handle;
    Result := True
  end else
    Result := False
end;

function TTntFrameBasePlugIn.GetObject(var APlugin: TTntFrame): Boolean;
begin
  APlugin := nil;
  beep;
  Result := True
end;

function TTntFrameBasePlugIn.HandleMessage(var Msg: TMsg;
  var Handled: Boolean): Boolean;
begin
  Handled := False;
  Result := True;
end;

function TTntFrameBasePlugIn.HintMsg(var AHint: WideString): Boolean;
begin
  Result := True;
  AHint := Hint
end;

function TTntFrameBasePlugIn.LoadConfig: Boolean;
begin
  Result := False
end;

function TTntFrameBasePlugIn.RegisterNotifier(ANotifier: IUltraNotifier): Boolean;
begin
  // Sends an valid IUltraNotifier when the plugin is created and a nil when
  // the Notifier Sink is being destroyed or the plugin is being destroyed
  Notifier := ANotifier;
  Result := True
end;

function TTntFrameBasePlugIn.SaveConfig: Boolean;
begin
  Result := False;
end;

function TTntFrameBasePlugIn.SetParentWnd(AParent: TWinControl): Boolean;
begin
  Parent := AParent;
  Align := alClient;
  Result := Parent = AParent
end;

function TTntFrameBasePlugIn.SetTheme(ThemeName: PChar): Boolean;
begin
  // Use if Form is in DLL where a new TBXSwitcher is needed
  Result := True
end;

function TTntFrameBasePlugIn.SetWindowBounds(AWidth, AHeight: Integer): Boolean;
begin
  Width := AWidth;
  Height := AHeight;
  Result := (AWidth = Width) and (AHeight = Height)
end;

function TTntFrameBasePlugIn.ShowWindow(Show: Boolean): Boolean;
begin
  Visible := Show;
  Result := Visible = Show
end;

function TTntFrameBasePlugIn.TabCaption(var ACaption: WideString): Boolean;
begin
  Result := True;
  ACaption := Caption
end;

function TTntFrameBasePlugIn._AddRef: Integer;
begin
  InterlockedIncrement(FRefCount);
  Result := RefCount;
end;

function TTntFrameBasePlugIn._Release: Integer;
begin
  InterlockedDecrement(FRefCount);
  Result := RefCount;
  if FRefCount = 0 then
    Free
end;

procedure TTntFrameBasePlugIn.TabChanging(TabState: TUltraInplaceNotifyTabState);
begin
 // case TabState of
 //   ntsCreating:    ShowMessage('Tab Creating');
 //   ntsActivating:  ShowMessage('Tab Activating');
 //   ntsDeActivating:ShowMessage('Tab Deactivating');
 //   ntsDestroying:  ShowMessage('Tab Destroying, Save your work');
 // end;
end;

function TTntFrameBasePlugIn.TabImage(var AnImageList: TCustomImageList;
  var AnImageIndex: Integer): boolean;
begin
  Result := False;
end;

procedure TTntFrameBasePlugIn.RootFolderChanged(Root: PItemIDList);
begin

end;

function TTntFrameBasePlugIn.QueryInterface(const IID: TGUID;
  out Obj): HResult;
begin
  if GetInterface(IID, Obj) then Result := S_OK
    else Result := E_NOINTERFACE
end;

end.
