unit UnitPlugInFlattenFolderFrame;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  TntForms,
  ShlObj,
  Dialogs,
  ImgList,
  StdCtrls,
  EasyListview,
  MPCommonObjects,
  MPCommonUtilities,
  MPShellUtilities,
  MPThreadManager,
  SpTBXControls,
  SpTBXDkPanels,
  SpTBXEditors,
  SpTBXItem,
  TntClasses,
  TntStdCtrls,
  UltraFavTree,
  UltraUserStrings,
  UltraUtilities,
  UnitPlugInBaseFrame,
  UnitInPlaceWndInf,
  VirtualExplorerEasyListview;

const
  WM_ENUMDONE = WM_APP + 100;
  ID_FILTER_BKGND = 1001;          // Resource ID

  FLATTEN_FOLDER_STREAM_VERSION = 0;
  
type
  TFlatFolderEnumThread = class(TCommonThread)
  private
    FItemList: TList;
    FOwnerWnd: TWinControl;
    FPathList: TList;
  protected
    procedure Execute; override;
    property ItemList: TList read FItemList write FItemList;
    property PathList: TList read FPathList write FPathList;
  public
    function EnumCallback(MessageWnd: HWnd; APIDL: PItemIDList; AParent: TNamespace; Data: Pointer; var Terminate: Boolean): Boolean;
    property OwnerWnd: TWinControl read FOwnerWnd write FOwnerWnd;
  end;


  TTntFramePlugInFlattenFolder = class(TTntFrameBasePlugIn)
    FlatFolderELV: TVirtualMultiPathExplorerEasyListview;
    ImageList: TImageList;
    procedure FlatFolderELVItemSelectionsChanged(Sender: TCustomEasyListview);
  private
    { Private declarations }
    FEnumThread: TFlatFolderEnumThread;
    FItemList: TList;
    FNamespaceList: TList;
    FShownOnce: boolean;
    FSpTBXDockHeader: TSpTBXDock;
    FSpTBXToolbar: TSpTBXToolbar;
  protected
    procedure WMEnumDone(var Msg: TMessage); message WM_ENUMDONE;
    property EnumThread: TFlatFolderEnumThread read FEnumThread write FEnumThread;
    property ItemList: TList read FItemList write FItemList;
    property ShownOnce: boolean read FShownOnce write FShownOnce;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
//    function IsActionEnabled(var InPlaceWindowAction: TInPlaceWindowActions): Boolean; override;
    function LoadConfig: Boolean; override;
    function SaveConfig: Boolean; override;
//    function SetQuickFilter(Enable: Boolean; FilterMask: WideString): Boolean; override;
    function SetTheme(ThemeName: PChar): Boolean; override;
    function ShowWindow(Show: Boolean): Boolean; override;
//    function StatusBarInfo(var ItemCount: Cardinal; var SelectedItemCount: Cardinal; var SelectedItemSize: Int64): Boolean; override;
    function TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean; override;
    property NamespaceList: TList read FNamespaceList write FNamespaceList;

    property SpTBXDockHeader: TSpTBXDock read FSpTBXDockHeader write FSpTBXDockHeader;
    property SpTBXToolbar: TSpTBXToolbar read FSpTBXToolbar write FSpTBXToolbar;
  end;

implementation

{$R *.dfm}

function FlattenFolderSettingsFile: WideString;
begin
  Result := AppFilesPath + '\' + STR_FLATTENFOLDER_FILE
end;

{ TFlatFolderEnumThread}

function TFlatFolderEnumThread.EnumCallback(MessageWnd: HWnd; APIDL: PItemIDList; AParent: TNamespace; Data: Pointer; var Terminate: Boolean): Boolean;
var
  NS: TNamespace;
begin
  NS := TNamespace.Create( PIDLMgr.AppendPIDL(AParent.AbsolutePIDL, APIDL), nil);
  if Assigned(NS) then
  begin
    if NS.Folder and NS.FileSystem then
    begin
      NS.EnumerateFolder(0, True, True, False, EnumCallback, Data);
      FreeAndNil(NS)
    end else
    begin
      if NS.FileSystem then
        TList( Data).Add(NS)
      else
        FreeAndNil(NS)
    end
  end;
  Terminate := Self.Terminated;
  Result := True;
end;

procedure TFlatFolderEnumThread.Execute;
var
  i: Integer;
  NS: TNamespace;
begin
  i := 0;
  while not Terminated and (i < PathList.Count) do
  begin
    NS := TNamespace(PathList[i]);
    NS.EnumerateFolder(0, True, True, False, EnumCallback, ItemList);
    Inc(i)
  end;
  if not Terminated and OwnerWnd.HandleAllocated then
    PostMessage(OwnerWnd.Handle, WM_ENUMDONE, 0, 0);
end;

{ TTntFramePlugInFlattenFolder }
constructor TTntFramePlugInFlattenFolder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TTntFramePlugInFlattenFolder.Destroy;
var
  i: Integer;
begin
  if Assigned(EnumThread) then
  begin
    EnumThread.Terminate;
    while not EnumThread.Finished do
      Sleep(100);
    FreeAndNil(FEnumThread);
  end;

  if Assigned(NamespaceList) then
  begin
    try
      for i := 0 to NamespaceList.Count - 1 do
        TObject(NamespaceList[i]).Free;
    finally
      NamespaceList.Count := 0;
      FreeAndNil(FNamespaceList);
    end
  end;
  if Assigned(ItemList) then
  begin
    try
      for i := 0 to ItemList.Count - 1 do
        TObject(ItemList[i]).Free;
    finally
      ItemList.Count := 0;
      FreeAndNil(FItemList);
    end
  end;

  inherited Destroy;
end;

function TTntFramePlugInFlattenFolder.LoadConfig: Boolean;
var
  F: TTntFileStream;
begin
  Result := True;
  try
    if WideFileExists(FlattenFolderSettingsFile) then
    begin
      F := TTntFileStream.Create(FlattenFolderSettingsFile, fmOpenRead or fmShareDenyWrite);
      try
        FlatFolderELV.Header.StreamColumns := True;
        FlatFolderELV.Header.LoadFromStream(F, FLATTEN_FOLDER_STREAM_VERSION)
      finally
        F.Free
      end
    end
  except
    Result := False
  end
end;

function TTntFramePlugInFlattenFolder.SaveConfig: Boolean;
var
  F: TTntFileStream;
begin
  Result := True;
  try
    F := TTntFileStream.Create(FlattenFolderSettingsFile, fmCreate or fmShareDenyWrite);
    try
      FlatFolderELV.Header.StreamColumns := True;
      FlatFolderELV.Header.SaveToStream(F, FLATTEN_FOLDER_STREAM_VERSION)
    finally
      F.Free
    end
  except
    Result := False
  end
end;

{
function TTntFramePlugInFlattenFolder.SetQuickFilter(Enable: Boolean; FilterMask: WideString): Boolean;
begin
  if Enabled then
    FlatFolderELV.QuickFilterMask := FilterMask;
    
  FlatFolderELV.QuickFiltered := Enable;
  FlatFolderELV.BackGround.Enabled := Enable;
  Result := True;
end;   }

function TTntFramePlugInFlattenFolder.SetTheme(ThemeName: PChar): Boolean;
begin
  Result:= inherited SetTheme(ThemeName);
  FlatFolderELV.Invalidate;
  Invalidate
end;

function TTntFramePlugInFlattenFolder.ShowWindow(Show: Boolean): Boolean;
begin
  Result:= inherited ShowWindow(Show);
  FixFormFont(Font);
  if Show and not ShownOnce then
  begin
    ItemList := TList.Create;
    EnumThread := TFlatFolderEnumThread.Create(True);
    EnumThread.PathList := NamespaceList;
    EnumThread.ItemList := ItemList;
    EnumThread.OwnerWnd := Self;
    EnumThread.Resume;
    FlatFolderELV.BackGround.Image.LoadFromResourceID(hInstance, ID_FILTER_BKGND);
    FlatFolderELV.BackGround.Transparent := True;
    FlatFolderELV.BackGround.OffsetTrack := True;
    FlatFolderELV.BackGround.AlphaBlend := True;
    FlatFolderELV.BackGround.Tile := False;
    FlatFolderELV.BackGround.BlendAlpha := 192;
    FlatFolderELV.BackGround.OffsetX := ClientWidth - 120;
    FlatFolderELV.BackGround.OffsetY := ClientHeight - 70;
    ShownOnce := True;
  end
end;

{
function TTntFramePlugInFlattenFolder.StatusBarInfo(var ItemCount: Cardinal; var SelectedItemCount: Cardinal; var SelectedItemSize: Int64): Boolean;
var
  Item: TExplorerItem;
begin
  Result := True;
  ItemCount := FlatFolderELV.ItemCount;
  SelectedItemCount := FlatFolderELV.Selection.Count;
  if SelectedItemSize = -1 then
  begin
    SelectedItemSize := 0;
    if Assigned(FlatFolderELV) then
    begin
      Item := FlatFolderELV.Selection.First as TExplorerItem;
      while Assigned(Item) do
      begin
        Inc(SelectedItemSize, Item.Namespace.SizeOfFileInt64);
        Item := FlatFolderELV.Selection.Next(Item) as TExplorerItem;
      end;
    end;
  end else
    SelectedItemSize := 0;
end;   }

function TTntFramePlugInFlattenFolder.TabImage(var AnImageList: TCustomImageList; var AnImageIndex: Integer): Boolean;
begin
  AnImageList := ImageList;
  AnImageIndex := 0;
  Result := True;
end;

procedure TTntFramePlugInFlattenFolder.WMEnumDone(var Msg: TMessage);
var
  i: Integer;
begin
  try
    FlatFolderELV.BeginUpdate;
    try
      for i := 0 to ItemList.Count - 1 do
        FlatFolderELV.AddCustomItem(nil, TNamespace( ItemList[i]), True);
    finally
      FlatFolderELV.EndUpdate();
      Notifier.UpdateUI(nuiStatusBar);
    end
  finally
    ItemList.Count := 0
  end
end;

procedure TTntFramePlugInFlattenFolder.FlatFolderELVItemSelectionsChanged(
  Sender: TCustomEasyListview);
begin
  Notifier.UpdateUI(nuiStatusBar);
  Notifier.UpdateUI(nuiActionbar);
end;

end.
